import os  # do tworzenia katalogów
import random
import sys  # do wysyłania komunikatów w czerwonym kolorze
import time  # dla testów wydajności
from fractions import Fraction

import numpy as np
import sympy as sp
from sympy import I
import warnings  # wyłącz ostrzeżenia przy generowaniu obrazów
warnings.filterwarnings('ignore')
random.seed()

# Todo: Poprawić/uzupełnić docstringi

def rownanie_liniowe(nr_warstwy: int = 0):  # na 10000 losowań było 9602 różnych

    # random.seed()
    z = sp.symbols('z', complex=True)
    wspolczynniki = [-6, -5, -4, -3, -2, -1, 2, 3, 4, 5, 6, 7, 8, 9]
    while True:
        a = [random.choice([2, 3, 4, 5, 6, 7, 8, 9]), random.choice(wspolczynniki)]
        b = [random.choice([2, 3, 4, 5, 6, 7, 8, 9]), random.choice(wspolczynniki)]
        c = [random.choice(wspolczynniki), random.choice(wspolczynniki)]
        rozwiazanie = sp.solve((a[0] + a[1] * I) * z - (b[0] + b[1] * I) * z - (c[0] + c[1] * I))
        if len(rozwiazanie) == 0:
            rozwiazanie = [0.5 + 0.5 * I]
        if (a[1] != b[1] and sp.re(rozwiazanie[0]) != 0 and sp.im(rozwiazanie[0]) != 0
                and int(sp.re(rozwiazanie[0])) == sp.re(rozwiazanie[0])
                and int(sp.im(rozwiazanie[0])) == sp.im(rozwiazanie[0])):
            break
    #    print(rozwiazanie)
    lewa_prawa = random.choice([0, 1])
    if lewa_prawa == 0:
        rownanie = str('\\left(' + str(sp.expand((a[0] + a[1] * I))) + '\\right)z = '
                       + str(sp.expand((c[0] + c[1] * I))) + '+\\left('
                       + str(sp.expand((b[0] + b[1] * I))) + '\\right)z')
    else:
        rownanie = str(str(sp.expand((c[0] + c[1] * I))) + '+\\left('
                       + str(sp.expand((b[0] + b[1] * I))) + '\\right)z = ' + '\\left('
                       + str(sp.expand((a[0] + a[1] * I))) + '\\right)z')

    rownanie = rownanie.replace('I', 'i')
    rownanie = rownanie.replace('*', '')
    rozwiazanie = str(rozwiazanie[0])
    rozwiazanie = rozwiazanie.replace('I', 'i')
    rozwiazanie = rozwiazanie.replace('*', '')
    return ('Rozwiązać równanie w zbiorze liczb zespolonych.\\ Sprawdzić rozwiązanie.\n'
            f'\t\\[\n'
            f'\t\t{rownanie}\n'
            f'\t\\]',
            f'$z={rozwiazanie}.$')
    # return [rownanie, list(rozwiazanie)]


def rownanie_kwadratowe(nr_warstwy: int = 0):  # na 2000 losowań było 194 różnych
    # random.seed()
    z = sp.symbols('z', complex=True)
    wspolczynniki = [-6, -5, -4, -3, -2, -1, 2, 3, 4, 5, 6, 7, 8, 9]
    while True:
        z1 = [random.choice(wspolczynniki), random.choice(wspolczynniki)]
        z2 = [random.choice(wspolczynniki), random.choice(wspolczynniki)]
        a = [random.choice([1, 2, 3, 4, 5, 6, 7, 8, 9]), random.choice(wspolczynniki)]
        delta = ((a[0] + a[1] * 1j) * (-(z1[0] + z1[1] * 1j)
                                       - (z2[0] + z2[1] * 1j))) ** 2 - 4 * (a[0] + a[1] * 1j) ** 2 * (
                        z1[0] + z1[1] * 1j) * (z2[0] + z2[1] * 1j)
        if (abs(delta) < 15
                and abs((a[0] + a[1] * 1j) * (-(z1[0] + z1[1] * 1j) - (z2[0] + z2[1] * 1j))) < 15
                and np.real(delta) != 0 and np.imag(delta) != 0
                and (a[0] * (z1[0] + z2[0]) - a[1] * (z1[1] + z2[1])) != 0
                and (a[1] * (z1[0] + z2[0]) + a[0] * (z1[1] + z2[1])) != 0):
            break
    sqrt_delta = sp.sqrt(int(sp.re(delta)) + int(sp.im(delta)) * I).expand(complex=True)
    przestawiony_trojmian = sp.collect(
        sp.expand((a[0] + a[1] * I) * (z - (z1[0] + z1[1] * I)) * (z - (z2[0] + z2[1] * I))), z)
    poprawiony_trojmian = str('\\left(' + str(przestawiony_trojmian.coeff(z, 2)) +
                              '\\right)z^2 + \\left(' + str(przestawiony_trojmian.coeff(z, 1)) +
                              '\\right) z + \\left(' + str(przestawiony_trojmian.coeff(z, 0)) + '\\right)')
    rozwiazanie = sp.solve(przestawiony_trojmian)
    poprawiony_trojmian = poprawiony_trojmian.replace('I', 'i')
    poprawiony_trojmian = poprawiony_trojmian.replace('*', ' ')
    rozwiazanie[0] = str(rozwiazanie[0])
    rozwiazanie[0] = rozwiazanie[0].replace('I', 'i')
    rozwiazanie[0] = rozwiazanie[0].replace('*', ' ')
    rozwiazanie[1] = str(rozwiazanie[1])
    rozwiazanie[1] = rozwiazanie[1].replace('I', 'i')
    rozwiazanie[1] = rozwiazanie[1].replace('*', ' ')
    delta = str(delta)
    delta = delta.replace('j', 'i')
    delta = delta.replace('(', '')
    delta = delta.replace(')', '')
    return (f'Rozwiązać równanie w zbiorze liczb zespolonych.\\ Sprawdzić jedno z rozwiązań.\n'
            f'\t\\[\n'
            f'\t\t{poprawiony_trojmian}=0\n'
            f'\t\\]',
            f'$\\Delta = {delta}, \\quad '
            f'\\sqrt{{\\Delta}}=\\pm( {sp.latex(sqrt_delta)}), \\quad '
            f'z_{{1}}={rozwiazanie[0]}, \\quad '
            f'z_{{2}}={rozwiazanie[1]}$')


# Todo: dołożyć wyrażenie w stylu ((1+i*sqrt(3))/2)^10 które wygeneruje 1 lub -1 lub
# Todo: 2 albo 3 do potęgi, która jest stopniem pierwiastka
def pierwiastek_zespolony(stopien=3, nr_warstwy=0):  # generuje 152 różne zadania z 3 i 4 stopniem
    if stopien == 3:
        liczba = random.choice([27 * I, -27 * I, 64 * I, -64 * I,
                                Fraction(64, 27) * I ** 4, -Fraction(64, 27) * I ** 4, Fraction(64, 27) * I,
                                -Fraction(64, 27) * I,
                                Fraction(27, 64) * I ** 4, -Fraction(27, 64) * I ** 4, Fraction(27, 64) * I,
                                -Fraction(27, 64) * I])
        # w powyższym bez I**4 pojawia się co jakiś czas błąd przy sp.latex(liczba) w odpowiedzi - nie wiem czemu.
        # liczba = (liczba**3).expand(complex=True).collect(I)
        while True:
            rozne_pierwiastki = random.choice([sp.sqrt(2), sp.sqrt(3), sp.sqrt(5)])
            a = random.choice((-1, 1))
            b = random.choice((-1, 1))
            iloraz = (liczba / (a + b * I * rozne_pierwiastki)).expand(complex=True).collect(I)
            if sp.im(iloraz) != 0 and sp.re(iloraz) != 0 and a != b and a != sp.im(iloraz):
                break
        z = sp.symbols('z', complex=True)
        rozwiazanie = sp.latex(sp.solve(z ** 3 - liczba, rational=True)).replace('[', '\{').replace(']', '\}')
        strona = random.choice((0, 1))  # do wymieszania nawiasów pod pierwiastkiem
        pierwiastek = (f'\t\\[\n'
                       f'\t\t\\sqrt[\\leftroot{{2}}\\uproot{{-4}} \\displaystyle ^{{3}}]{{\\left('
                       f'{sp.latex(iloraz) if strona == 0 else sp.latex(a + b * I * rozne_pierwiastki)} \\right)'
                       f' \\left({sp.latex(a + b * I * rozne_pierwiastki) if strona == 0 else sp.latex(iloraz)} \\right)}}\n'
                       f'\t\\]')
        return ('Wyznaczyć wszystkie zadane pierwiastki zespolone i zaznaczyć je na płaszczyźnie zespolonej\n'
                f'{pierwiastek}',
                f'$\\sqrt[\\leftroot{{2}}\\uproot{{-4}} \\displaystyle ^{{3}}]{{{sp.latex(liczba)}}} = {rozwiazanie}.$')
    if stopien == 4:
        liczba = random.choice(
            [1 + sp.sqrt(3) * I, 1 - sp.sqrt(3) * I, -1 + sp.sqrt(3) * I, -1 - sp.sqrt(3) * I,
             sp.sqrt(3) + I, sp.sqrt(3) - I, -sp.sqrt(3) + I, -sp.sqrt(3) - I,
             1 + I, 1 - I, -1 + I, -1 - I])  # te ostatnie generują łatwe liczby pod pierwiastkiem
        # print(liczba)
        liczba = (liczba ** 4).expand(complex=True).collect(I)
        while True:
            rozne_pierwiastki = random.choice([sp.sqrt(2), sp.sqrt(3), sp.sqrt(5)])
            a = random.choice((-2, -1, 1, 2))
            b = random.choice((-2, -1, 1, 2))
            iloraz = (liczba / (a + b * I * rozne_pierwiastki)).expand(complex=True).collect(I)
            if sp.im(iloraz) != 0 and sp.re(iloraz) != 0 and a != b and a != sp.im(iloraz):
                break
        # print(liczba)
        z = sp.symbols('z', complex=True)
        rozwiazanie = sp.latex(sp.solve(z ** 4 - liczba)).replace('[', '\{').replace(']', '\}')
        # print(sp.latex(rozwiazanie))
        strona = random.choice((0, 1))  # do wymieszania nawiasów pod pierwiastkiem
        pierwiastek = (f'\t\\[\n'
                       f'\t\t\\sqrt[\\leftroot{{2}}\\uproot{{-4}} \\displaystyle ^{{4}}]{{\\left('
                       f'{sp.latex(iloraz) if strona == 0 else sp.latex(a + b * I * rozne_pierwiastki)} \\right)'
                       f' \\left({sp.latex(a + b * I * rozne_pierwiastki) if strona == 0 else sp.latex(iloraz)} \\right)}}\n'
                       f'\t\\]')
        # pierwiastek = str('$$\\sqrt[\\leftroot{2}\\uproot{-4} \\displaystyle ^{4}]{\\left(' + sp.latex(
        #     iloraz) + '\\right) \\left(' + sp.latex(a + b * I * rozne_pierwiastki) + '\\right)}$$')
        return ('Wyznaczyć wszystkie zadane pierwiastki zespolone i zaznaczyć je na płaszczyźnie zespolonej\n'
                f'{pierwiastek}',
                f'$\\sqrt[\\leftroot{{2}}\\uproot{{-4}} \\displaystyle ^{{4}}]{{{sp.latex(liczba)}}} = {rozwiazanie}.$')
        # return str(
        #     r'        \item Wyznaczyć wszystkie zadane pierwiastki zespolone i zaznaczyć je na płaszczyźnie zespolonej.' + '\n' +
        #     '        ' + pierwiastek + '\n' +
        #     '        \\textcolor{\kolorodpowiedzi}{Rozwiązaniem jest $z = ' + sp.latex(rozwiazanie).replace('[',
        #                                                                                                      '\{').replace(
        #         ']', '\}') + '.$}\n\n'
        # )


def rownanie_ze_sprzezeniem(calkowite: bool = 1, kwadratowe: bool = 1):
    x, y = sp.symbols('x y', real=True)
    z = sp.symbols('z', complex=True)
    wspolczynniki = (-6, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 6, 7)
    while True:
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        for i in range(4):  # losuje 3 liczb zespolonych i zapisuje w słowniku
            [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
            liczby[i] = u + v * I
        r = x + I * y  # inne oznaczenie by osobno wyświetlać równanie z "z" oraz równanie z "x" i "y"
        rownanie = ((liczby[0] if kwadratowe is False else sp.re(liczby[0]))
                    + (liczby[1] if kwadratowe is False else sp.re(liczby[1])) * sp.conjugate(z)
                    + (1 - int(kwadratowe)) * liczby[2] * z + int(kwadratowe) * z ** 2)
        rownanie_latex = sp.latex(rownanie)
        # rownanie = ((liczby[0] if kwadratowe == 0 else sp.re(liczby[0]))
        #             + (liczby[1] if kwadratowe == 0 else sp.re(liczby[1])) * sp.conjugate(r)
        #             + (1 - kwadratowe) * liczby[2] * r + kwadratowe * r ** 2)
        rownanie = rownanie.subs(z, r)
        rozwiazanie = sp.solve(rownanie)
        try:
            if len(rozwiazanie) > 0 and (type(rozwiazanie) == list and all(
                    (all if calkowite else any)((int(i) == i and -10 < i < 10) for i in j.values()) for j in
                    rozwiazanie)
                                         or type(rozwiazanie) == dict and (all if calkowite else any)(
                        (int(i) == i and -10 < i < 10) for i in rozwiazanie.values())):
                break
        except Exception:
            print(Exception)
            break
    # print(print([rozwiazanie[i] for i in rozwiazanie.keys()]))
    # print(rownanie_latex)
    # print(sp.latex(rownanie))
    # print(sp.latex(sp.re(rownanie)))
    # print(sp.latex(sp.im(rownanie)))
    # print(rozwiazanie)
    return (
        f'Rozwiązać równanie w zbiorze liczb zespolonych\n'
        f'\t\\[\n'
        f'\t\t{rownanie_latex} = 0\n'
        f'\t\\]',
        f'${sp.latex(rownanie)} = 0,$ \\\\ \n'
        f'${sp.latex(rownanie.factor(x, y))} = 0,$\\\\\n'
        f'$\\left\\{{\n'
        f'\t\\begin{{array}}{{c}}\n'
        f'\t\t{sp.latex(sp.re(rownanie))} = 0\\\\\n'
        f'\t\t{sp.latex(sp.im(rownanie))} = 0\n'
        f'\t\\end{{array}}\n'
        f'\\right.$ \\\\\n'
        f'$z = {sp.latex(rozwiazanie)}.$')

    # print(sp.solve(z ** 5 - I, (x, y)))  # trudne
    # print(sp.solve(z ** 3 + sp.Abs(z) ** 2 * sp.conjugate(z), (x, y)))  # trudne
    # print(sp.solve(z + sp.conjugate(z) - sp.Abs(z), (x, y)))
    # print(sp.solve(z ** 2 + 2 * sp.conjugate(z), (x, y)))
    # print(sp.solve(2 * z + (4 - I) * sp.conjugate(z) - 15 - 4 * I, (x, y)))


def obszar_zespolony(typ: int = 1, nr_zadania: int = 1):
    # https://stackoverflow.com/questions/40747474/how-to-customize-and-add-extra-elements-to-sympy-plots
    if not os.path.exists('pics'):
        os.makedirs('pics')
        print(" ! Tworzę katalog pics ", file=sys.stderr)
    x, y = sp.symbols('x y', real=True)
    z = x + I * y
    from spb import plot_implicit
    # from sympy import plot_implicit, symbols, Eq, And, Or
    if typ == 1:  # przesunięty okrąg
        wspolczynniki = (-3, -2, -1, 1, 2, 3)
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
        liczby[0] = u + v * I
        promien = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
        warunek = sp.Abs(z - liczby[0]) - promien
        warunki = ('\\left|z'
                   + ('+' if sp.re(liczby[0]) < 0 else '')
                   + (f'{sp.latex(-liczby[0])}\\right|  =  {sp.latex(promien)}'))
        p1 = plot_implicit(
            warunek,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            show=False,
            # points=1000,
            aspect='equal',
            size=(4, 4),
            markers=[{'args': [sp.re(liczby[0]), sp.im(liczby[0])], 'color': "black", 'marker': "o", 'ms': 3}
                     # , {'args': [[sp.re(liczby[0]), sp.re(liczby[0])+promien], [sp.im(liczby[0]), sp.im(liczby[0])]],
                     #    'color': "blue", 'ls': '-', 'lw': 2}
                     ],
            # annotations=[{'xy': (sp.re(liczby[0]), sp.im(liczby[0])), 'text': "  S", 'ha': 'left', 'va': 'top', 'color': 'blue'},
            #              {'xy': (2, 2), 'text': "x=y ", 'ha': 'right', 'va': 'bottom', 'color': 'blue'}]
        )
        #
        # warunek = sp.latex(sp.Abs(z - liczby[0])) + ' = ' + f'{sp.latex(promien)}'
        warunek = sp.latex(
            (x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2) + ' = ' + f'{sp.latex(promien ** 2)}'
    if typ == 2:  # przesunięte koło - wnętrze lub zewnętrze
        wspolczynniki = (-3, -2, -1, 1, 2, 3)
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
        liczby[0] = u + v * I
        wieksze_mniejsze = random.choice(('>', r'\geq', '<', r'\leq'))
        promien = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
        warunek = sp.Abs(
            z - (liczby[0])) > promien if wieksze_mniejsze == '>' or wieksze_mniejsze == r'\geq' else sp.Abs(
            z - (liczby[0])) < promien
        warunki = ('\\left|z' + (
            '+' if sp.re(liczby[0]) < 0 else '') + f'{sp.latex(-liczby[0])}\\right|  ' +
                   wieksze_mniejsze +
                   f' {sp.latex(promien)}')
        p1 = plot_implicit(
            warunek,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            show=False,
            # points=1000,
            aspect='equal',
            size=(4, 4),
            markers=[{'args': [sp.re(liczby[0]), sp.im(liczby[0])], 'color': "black", 'marker': "o", 'ms': 3}
                     # , {'args': [[sp.re(liczby[0]), sp.re(liczby[0])+promien], [sp.im(liczby[0]), sp.im(liczby[0])]],
                     #    'color': "blue", 'ls': '-', 'lw': 2}
                     ],
            # annotations=[{'xy': (sp.re(liczby[0]), sp.im(liczby[0])), 'text': "  S", 'ha': 'left', 'va': 'top', 'color': 'blue'},
            #              {'xy': (2, 2), 'text': "x=y ", 'ha': 'right', 'va': 'bottom', 'color': 'blue'}]
        )
        # tu były cyrki spowodowane chyba brakiem wymiarów przy drugim wykresie.
        warunek = sp.Abs(z - liczby[0]) - promien
        p2 = plot_implicit(
            warunek,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            # points=1000,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"linestyles": ":" if wieksze_mniejsze == '>' or wieksze_mniejsze == '<' else "-",
                          "linewidths": 2}
        )

        p1.append(p2[0])
        # warunek = sp.latex(warunek)
        warunek = (f'{sp.latex((x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2)}' +
                   wieksze_mniejsze +
                   f'{sp.latex(promien ** 2)}')

    if typ == 3:  # pierścień
        wspolczynniki = (-3, -2, -1, 1, 2, 3)
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
        liczby[0] = u + v * I
        while True:
            promien1 = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
            promien2 = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
            if promien1 < promien2 - 0.2:  # przy promieniach sqrt(2) i 3/2 nie widać pierścienia
                break
        nierownosc_mniejsza = random.choice(['>', r'\geq'])
        nierownosc_wieksza = random.choice(['<', r'\leq'])
        warunek1 = sp.Abs(z - liczby[0]) > promien1
        warunek2 = sp.Abs(z - liczby[0]) < promien2
        warunki = (f'{sp.latex(promien1)}' +
                   ('<' if nierownosc_mniejsza == '>' else r'\leq') +
                   f' \\left|z' + ('+' if sp.re(liczby[0]) < 0 else '') +
                   f'{sp.latex(-liczby[0])}\\right| {nierownosc_wieksza} {sp.latex(promien2)}')

        p1 = plot_implicit(sp.And(
            warunek1,
            warunek2
        ),
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            show=False,
            adaptive=False,
            depth=2,  # dokladność rysowania obszarów - od 0 do 4 - 4 bardzo czasochłonne
            # i generuje duże pliki - problem z kompilacją i potrzebny lualatex, który z kolei zgłasza dodatkowe problemy.
            aspect='equal',
            size=(4, 4),
            markers=[
                {'args': [sp.re(liczby[0]), sp.im(liczby[0])], 'color': "black", 'marker': "o", 'ms': 3}
                # , {'args': [[sp.re(liczby[0]), sp.re(liczby[0])+promien], [sp.im(liczby[0]), sp.im(liczby[0])]],
                #    'color': "blue", 'ls': '-', 'lw': 2}
            ],
            # annotations=[{'xy': (sp.re(liczby[0]), sp.im(liczby[0])), 'text': "  S", 'ha': 'left', 'va': 'top', 'color': 'blue'},
            #              {'xy': (2, 2), 'text': "x=y ", 'ha': 'right', 'va': 'bottom', 'color': 'blue'}]
        )

        p2 = plot_implicit(
            # sp.geometry.Circle(sp.geometry.Point(sp.re(liczby[0]), sp.im(liczby[0])), promien1).equation(),
            sp.Abs(z - liczby[0]) - promien1,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            # points=1000,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"linestyles": ":" if nierownosc_mniejsza == '>' else "-",
                          "linewidths": 1}
        )
        p1.append(p2[0])
        p3 = plot_implicit(
            sp.Abs(z - liczby[0]) - promien2,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"linestyles": ":" if nierownosc_wieksza == '<' else "-",
                          "linewidths": 1}
        )
        p1.append(p3[0])

        warunek1 = (f'{sp.latex((x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2)}' +
                    nierownosc_mniejsza +
                    f'{sp.latex(promien1 ** 2)}')
        warunek2 = (f'{sp.latex((x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2)}' +
                    nierownosc_wieksza +
                    f'{sp.latex(promien2 ** 2)}')
        warunek = warunek1 + '\\\\\n\t\t' + warunek2
    if typ == 4:  # przesunięte koła o różnych środkach (wnętrza lub zewnętrzna)
        wspolczynniki = (-3, -2, -1, 1, 2, 3)
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        przeciecie = random.choice((True, False))
        while True:
            [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
            liczby[0] = u + v * I
            [u, v] = [random.choice(wspolczynniki) for _ in range(2)]
            liczby[1] = u + v * I
            promien1 = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
            promien2 = random.choice([sp.Rational(1 / 2), 1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3])
            if (przeciecie is True
                    and liczby[0] != liczby[1]
                    and sp.Abs(promien1 - promien2) < sp.Abs(liczby[0] - liczby[1]) < promien1 + promien2):
                break
            if przeciecie is False:
                break
        nierownosc_pierwsza = random.choice(['>', r'\geq', '<', r'\leq'])
        nierownosc_druga = random.choice(['>', r'\geq', '<', r'\leq'])
        warunek1 = sp.Abs(z - liczby[0]) < promien1 if nierownosc_pierwsza in ('<', r'\leq') else sp.Abs(
            z - liczby[0]) > promien1
        warunek2 = sp.Abs(z - liczby[1]) < promien2 if nierownosc_druga in ('<', r'\leq') else sp.Abs(
            z - liczby[1]) > promien2
        warunki = ('\\left|z' + (
            '+' if sp.re(
                liczby[0]) < 0 else '') + f'{sp.latex(-liczby[0])}\\right| {nierownosc_pierwsza} {sp.latex(promien1)}\n'
                                          f' \t\t\\quad \\textnormal{{oraz}} \\quad\n\t\t'
                                          '\\left|z' + (
                       '+' if sp.re(liczby[
                                        1]) < 0 else '') + f'{sp.latex(-liczby[1])}\\right| {nierownosc_druga} {sp.latex(promien2)}')
        p1 = plot_implicit(sp.And(
            warunek1
        ),
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            show=False,
            adaptive=False,
            depth=2,  # dokladność rysowania obszarów - od 0 do 4 - 4 bardzo czasochłonne
            # i generuje duże pliki - problem z kompilacją i potrzebny lualatex, który z kolei zgłasza dodatkowe problemy.
            aspect='equal',
            size=(4, 4),
            markers=[
                {'args': [sp.re(liczby[0]), sp.im(liczby[0]), sp.re(liczby[1]), sp.im(liczby[1])], 'color': "black",
                 'marker': "o", 'ms': 3}
                # , {'args': [[sp.re(liczby[0]), sp.re(liczby[0])+promien], [sp.im(liczby[0]), sp.im(liczby[0])]],
                #    'color': "blue", 'ls': '-', 'lw': 2}
            ],
            rendering_kw={"alpha": 0.1}
            # annotations=[{'xy': (sp.re(liczby[0]), sp.im(liczby[0])), 'text': "  S", 'ha': 'left', 'va': 'top', 'color': 'blue'},
            #              {'xy': (2, 2), 'text': "x=y ", 'ha': 'right', 'va': 'bottom', 'color': 'blue'}]
        )
        p2 = plot_implicit(sp.And(
            warunek2
        ),
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            show=False,
            adaptive=False,
            depth=2,  # dokladność rysowania obszarów - od 0 do 4 - 4 bardzo czasochłonne
            # i generuje duże pliki - problem z kompilacją i potrzebny lualatex, który z kolei zgłasza dodatkowe problemy.
            aspect='equal',
            size=(4, 4),
            markers=[
                {'args': [sp.re(liczby[0]), sp.im(liczby[0]), sp.re(liczby[1]), sp.im(liczby[1])], 'color': "black",
                 'marker': "o", 'ms': 3}
                # , {'args': [[sp.re(liczby[0]), sp.re(liczby[0])+promien], [sp.im(liczby[0]), sp.im(liczby[0])]],
                #    'color': "blue", 'ls': '-', 'lw': 2}
            ],
            rendering_kw={"alpha": 0.1}
            # annotations=[{'xy': (sp.re(liczby[0]), sp.im(liczby[0])), 'text': "  S", 'ha': 'left', 'va': 'top', 'color': 'blue'},
            #              {'xy': (2, 2), 'text': "x=y ", 'ha': 'right', 'va': 'bottom', 'color': 'blue'}]
        )
        p1.append(p2[0])
        p3 = plot_implicit(sp.And(warunek1,
                                  warunek2
                                  ),
                           (x, -7, 7),
                           (y, -7, 7),
                           show_in_legend=False,
                           show=False,
                           adaptive=False,
                           depth=3,  # dokladność rysowania obszarów - od 0 do 4 - 4 bardzo czasochłonne
                           # i generuje duże pliki - problem z kompilacją i potrzebny lualatex, który z kolei zgłasza dodatkowe problemy.
                           aspect='equal',
                           size=(4, 4),
                           rendering_kw={"alpha": 0.9}
                           )
        p1.append(p3[0])

        p4 = plot_implicit(
            # sp.geometry.Circle(sp.geometry.Point(sp.re(liczby[0]), sp.im(liczby[0])), promien1).equation(),
            sp.Abs(z - liczby[0]) - promien1,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            depth=2,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"alpha": 0.2, "linestyles": ":" if nierownosc_pierwsza in ('<', '>') else "-",
                          "linewidths": 1}
        )
        p1.append(p4[0])
        p5 = plot_implicit(
            sp.Abs(z - liczby[1]) - promien2,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            depth=2,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"alpha": 0.2, "linestyles": ":" if nierownosc_druga in ('<', '>') else "-",
                          "linewidths": 1}
        )
        p1.append(p5[0])

        warunek1 = (f'{sp.latex((x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2)} ' +
                    nierownosc_pierwsza +
                    f' {sp.latex(promien1 ** 2)}')
        warunek2 = (f'{sp.latex((x - sp.re(liczby[1])) ** 2 + (y - sp.im(liczby[1])) ** 2)} ' +
                    nierownosc_druga +
                    f' {sp.latex(promien2 ** 2)}')
        warunek = warunek1 + '\\\\\n\t\t' + warunek2

    # strasznie mi się poniższe nie podoba (koszmar) - ale nie będzie mi się chciało tego poprawiać.
    if typ == 5:  # pierścień w środku układu plus argument.
        argument1_nierownosc = random.choice(('>', r'\geq'))
        argument2_nierownosc = random.choice(('<', r'\leq'))
        katy = ('-5 / 6', '-3 / 4', '-2 / 3', '1 / 2', '-1 / 3', '-1 / 4', '-1 / 6',
                '1', '5 / 6', '3 / 4', '2 / 3', '1 / 2', '1 / 3', '1 / 4', '1 / 6')
        while True:
            phi1 = sp.Rational(random.choice(katy)) * sp.pi
            phi2 = sp.Rational(random.choice(katy)) * sp.pi
            if phi1 < phi2:
                break
        liczby = {}  # potencjalnie lepsze od tworzenia listy i dodawania elementów
        liczby[0] = 0
        while True:
            promien1 = random.choice([1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3, 4, 5])
            promien2 = random.choice([1, sp.sqrt(2), sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3, 4, 5])
            if promien1 < promien2 - 0.2:  # przy promieniach sqrt(2) i 3/2 nie widać pierścienia
                break
        nierownosc_mniejsza = random.choice(['>', r'\geq'])
        nierownosc_wieksza = random.choice(['<', r'\leq'])
        warunek1 = sp.Abs(z - liczby[0]) > promien1
        warunek2 = sp.Abs(z - liczby[0]) < promien2

        warunki = (f'{sp.latex(promien1)}' +
                   ('<' if nierownosc_mniejsza == '>' else r'\leq') +
                   f' \\left|z \\right| {nierownosc_wieksza} {sp.latex(promien2)}\n'
                   f' \t\t\\quad \\textnormal{{oraz}} \\quad\n\t\t'
                   f'{sp.latex(phi1)} ' +
                   ('<' if argument1_nierownosc == '>' else r'\leq') +
                   f' \\arg(z) ' +
                   ('<' if argument2_nierownosc == '<' else r'\leq') +
                   f' {sp.latex(phi2)} ')

        p1 = plot_implicit((x) ** 2 + y ** 2,
                           (x, -7, 7),
                           (y, -7, 7),
                           show_in_legend=False,
                           show=False,
                           aspect='equal',
                           size=(4, 4),
                           markers=[
                               {'args': [[0, 10 * sp.cos(phi1)], [0, 10 * sp.sin(phi1)]],
                                # 'color': "blue",
                                'ls': (':' if argument1_nierownosc in ('>') else "-"),
                                'lw': 1}],
                           rendering_kw={"alpha": 0.2},
                           )
        p2 = plot_implicit((x) ** 2 + y ** 2,
                           (x, -7, 7),
                           (y, -7, 7),
                           show_in_legend=False,
                           show=False,
                           aspect='equal',
                           size=(4, 4),
                           markers=[
                               {'args': [[0, 10 * sp.cos(phi2)], [0, 10 * sp.sin(phi2)]],
                                # 'color': "blue",
                                'ls': (':' if argument2_nierownosc in ('<') else "-"),
                                'lw': 1}],
                           rendering_kw={"alpha": 0.2},
                           )
        p1.append(p2[1])  # dlaczego p2[1]?

        p3 = plot_implicit(sp.And((sp.And if phi2 - phi1 < sp.pi else sp.Or)(y * sp.cos(phi1) > sp.sin(phi1) * x,
                                                                             y * sp.cos(phi2) < sp.sin(phi2) * x),
                                  warunek1, warunek2),
                           (x, -7, 7),
                           (y, -7, 7),
                           depth=2,
                           show_in_legend=False,
                           show=False,
                           rendering_kw={"alpha": 0.9},
                           aspect='equal',
                           size=(4, 4),
                           )
        p1.append(p3[0])

        p4 = plot_implicit(
            # sp.geometry.Circle(sp.geometry.Point(sp.re(liczby[0]), sp.im(liczby[0])), promien1).equation(),
            sp.Abs(z - liczby[0]) - promien1,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            # points=1000,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"alpha": 0.2, "linestyles": ":" if nierownosc_mniejsza == '>' else "-",
                          "linewidths": 1}
        )
        p1.append(p4[0])
        p5 = plot_implicit(
            sp.Abs(z - liczby[0]) - promien2,
            (x, -7, 7),
            (y, -7, 7),
            show_in_legend=False,
            color="red",
            show=False,
            aspect='equal',
            size=(4, 4),
            rendering_kw={"alpha": 0.2, "linestyles": ":" if nierownosc_wieksza == '<' else "-",
                          "linewidths": 1}
        )
        p1.append(p5[0])

        warunek1 = (f'{sp.latex(promien1 ** 2)}' +
                    (' < ' if nierownosc_mniejsza == '>' else r'\leq ') +
                    f'{sp.latex((x - sp.re(liczby[0])) ** 2 + (y - sp.im(liczby[0])) ** 2)}' +
                    nierownosc_wieksza +
                    f'{sp.latex(promien2 ** 2)}')
        warunek3 = (f'{sp.latex(phi1)} ' +
                    ('<' if argument1_nierownosc == '>' else r'\leq') +
                    f' \\varphi ' +
                    ('<' if argument2_nierownosc == '<' else r'\leq') +
                    f' {sp.latex(phi2)} ')
        warunek = warunek1 + '\\\\\n\t\t' + warunek3

    p1.xlabel = None
    p1.ylabel = None
    p1.save(f'./pics/obszar{nr_zadania}.pdf')
    p1.save(f'./pics/obszar{nr_zadania}.png')
    # p1.save(f'./pics/obszar{nr_zadania}.pgf')  # importujemy pdf-y bo pgf są jakieś ogromne przy obszarach zespolonych
    # print(sp.latex(warunek1), sp.latex(warunek2))
    p1.close()
    if typ < 3:  # do wyrównania warunków przy odpowiedziach
        przesuniecie = -2.8
    elif typ < 5:
        przesuniecie = -3.1
    else:
        przesuniecie = -3.3
    return ('Zaznaczyć na płaszczyźnie zespolonej obszar spełniający ' + (
        'warunek' if typ < 4 else 'warunki') + '\n' +
            f'\t\[\n'
            f'\t\t{warunki}\n'
            f'\t\]',
            f'\t\\begin{{tabular}}{{p{{0.4\\textwidth}}p{{0.3\\textwidth}}}}\n'
            f'\t\\begin{{gather*}}\n'
            f'\t\t{warunek}\n'
            f'\t\\end{{gather*}}'
            f'\t\t\n'
            f'\t&\n'
            # f'\t\t\\raisebox{{{przesuniecie}cm}}{{\\resizebox{{4cm}}{{!}}{{\\input{{../pics/obszar{nr_zadania}.pgf}}}}}}\n'
            f'\t\t\\raisebox{{{przesuniecie}cm}}{{\\resizebox{{4cm}}{{!}}{{\\includegraphics{{../pics/obszar{nr_zadania}}}}}}}\n'
            f'\t\\end{{tabular}}\n')


if __name__ == "__main__":  # to się uruchamia tylko, gdy plik jest uruchamiany jako program, a nie ładowany jako moduł
    start_time = time.time()
    # print('!!! zespolone.py były planowane jako moduł a uruchomiłeś je jako skrypt !!!')
    # polecenie, rozwiazanie = rownanie_liniowe()
    # print(polecenie,'\n', rozwiazanie)
    # polecenie, rozwiazanie = rownanie_kwadratowe()
    # polecenie, rozwiazanie = pierwiastek_zespolony(stopien=random.choice([3, 4]))
    # polecenie, rozwiazanie = rownanie_ze_sprzezeniem(calkowite=random.choice([0, 1]), kwadratowe=random.choice([0, 1]))
    for i in range(1):
        polecenie, rozwiazanie = obszar_zespolony(typ=5, nr_zadania=i)
        print(polecenie, '\n', rozwiazanie)
    print("Czas generowania --- %s seconds ---" % (time.time() - start_time))



import os
import random
import sys  # do wysyłania komunikatów w czerwonym kolorze
import time

''' Następne linijki likwidują konflikt z innymi modułami 
Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
import matplotlib.pyplot as plt
''' Poprzednie linijki likwidują konflikt z innymi modułami 
Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
import numpy as np
import sympy as sp
from matplotlib.backends.backend_pdf import PdfPages
from spb import plot_piecewise




def szereg_Fouriera(typ_l: int = 1, typ_p: int = 1, bez_wykresu: bool = False, nr_zadania: int = 1):
    ''' Następne linijki likwidują konflikt z innymi modułami
    Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
    Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
    plt.rc('text', usetex=True)
    plt.rc('font', family='serif')
    ''' Poprzednie linijki likwidują konflikt z innymi modułami 
    Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
    Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''

    if bez_wykresu is not True and not os.path.exists('pics'):
        os.makedirs('pics')
        print(" ! Tworzę katalog pics ", file=sys.stderr)
    x = sp.Symbol('x', Real=True)
    n = sp.symbols('n', integer=True, positive=True)
    okres_z_pi = random.choice([False, True])

    if typ_l == 0:  # by nie wypadła funkcja stała
        if typ_p == 0:
            while True:
                a = random.randint(-2, 2)
                b = random.randint(-2, 2)
                if a != b:
                    f_left = a * x / x
                    f_right = b * x / x
                    break
        else:
            a = random.randint(-2, 2)
            f_left = a * x / x

    if typ_l == 1:  # by nie wypadła funkcja stała
        f_left = (-1) ** random.randint(0, 1) * random.randint(1, 2) * x + random.randint(-2, 2) * x / x
    if typ_l == 2:
        f_left = (-1) ** random.randint(0, 1) * x ** 2
    if typ_l == 3:
        f_left = (-1) ** random.randint(0, 1) * sp.sin(x)
    if typ_l == 4:
        f_left = (-1) ** random.randint(0, 1) * sp.cos(x)
    if typ_p == 0:  # by nie wypadła funkcja stała
        if typ_l == 0:
            while True:
                a = random.randint(-2, 2)
                b = random.randint(-2, 2)
                if a != b:
                    f_left = a * x / x
                    f_right = b * x / x
                    break
        else:
            b = random.randint(-2, 2)
            f_right = b * x / x
    if typ_p == 1:  # by nie wypadła funkcja stała
        f_right = (-1) ** random.randint(0, 1) * random.randint(1, 2) * x + random.randint(-2, 2) * x / x
    if typ_p == 2:
        f_right = (-1) ** random.randint(0, 1) * x ** 2
    if typ_p == 3:
        f_right = (-1) ** random.randint(0, 1) * sp.sin(x)
    if typ_p == 4:
        f_right = (-1) ** random.randint(0, 1) * sp.cos(x)
    T = random.randint(1, 2) if okres_z_pi is False else random.randint(1, 3) * sp.pi / 2

    lewostronnie_0 = random.choice([False, True])
    lewostronnie_T = random.choice([False, True])
    # print(typ_l, f_left, typ_p, f_right, lewostronnie_0, lewostronnie_T)
    funkcja = [[(f_left.subs(x, x - 2 * i * T), (x < (2 * i) * T if lewostronnie_0 else x <= (2 * i) * T)),
                (f_right.subs(x, x - 2 * i * T), (x < (2 * i + 1) * T if lewostronnie_T else x <= (2 * i + 1) * T))] for
               i in range(-9, 10, 1)]
    f = sp.Piecewise(*sum(funkcja, []))

    a0 = (
        T ** (-1) * sp.integrate(f, (x, -T, T)).simplify() if okres_z_pi else sp.Rational(1, T) * sp.integrate(f, (
            x, -T, T)).simplify())
    an = (
        T ** (-1) * sp.integrate(f * sp.cos(n * sp.pi * x / T),
                                 (x, -T, T)).simplify() if okres_z_pi else sp.Rational(1,
                                                                                       T) * sp.integrate(
            f * sp.cos(n * sp.pi * x / T), (x, -T, T)).simplify())
    # n = sp.symbols('n', integer=True, positive=True)
    bn = (
        T ** (-1) * sp.integrate(f * sp.sin(n * sp.pi * x / T),
                                 (x, -T, T)).simplify() if okres_z_pi else sp.Rational(1,
                                                                                       T) * sp.integrate(
            f * sp.sin(n * sp.pi * x / T), (x, -T, T)).simplify())



    def F_n(ile_wyrazow: int):
        return a0 / 2 + sum(
            [an.subs(n, i) * sp.cos(i * sp.pi * x / T) + bn.subs(n, i) * sp.sin(i * sp.pi * x / T) for i in
             range(1, ile_wyrazow + 1)])
    if bez_wykresu is not True:
        with PdfPages(f'./pics/szereg_Fouriera_{nr_zadania}.pdf') as pdf:
            wykres = plot_piecewise(f, {"alpha": 0},  # niewidzialny dla pierwszej strony w odpowiedziach
                                    xlim=(
                                        (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                            -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                    size=(10, 2.5),
                                    adaptive=False,
                                    nb_of_points=1000,
                                    dots=True,
                                    legend=True,
                                    label=[f"$f(x)$"],
                                    ylabel='',
                                    title=f'Funkcja $f(x)$',
                                    show=False)
            ax = wykres.ax
            plt.legend(loc='center right')
            if okres_z_pi:
                ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                ax.set_xticks(list(ax.get_xticks()),
                              ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                               "$-\\frac{1}{2}\pi$",
                               "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                               '$3\pi$']);
            pdf.savefig(bbox_inches="tight")
            wykres.close()
            wykres = plot_piecewise(f,
                                    xlim=(
                                        (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                            -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                    size=(10, 2.5),
                                    adaptive=False,
                                    nb_of_points=1000,
                                    dots=True,
                                    legend=True,
                                    label=[f"$f(x)$"],
                                    ylabel='',
                                    title=f'Funkcja $f(x)$',
                                    show=False)
            ax = wykres.ax
            plt.legend(loc='center right')
            if okres_z_pi:
                ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                ax.set_xticks(list(ax.get_xticks()),
                              ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                               "$-\\frac{1}{2}\pi$",
                               "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                               '$3\pi$']);
            # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_funkcja.pdf', dpi=300, bbox_inches="tight")
            pdf.savefig(bbox_inches="tight")
            wykres.close()

            # if a0 != 0:
            wykres = plot_piecewise(f, a0 / 2,
                                    xlim=(
                                        (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                            -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                    size=(10, 2.5),
                                    nb_of_points=1000,
                                    dots=True,
                                    legend=True,
                                    label=["$f(x)$", "$\\frac{a_0}{2}$"],
                                    ylabel='',
                                    title=f'Funkcja $f(x)$ i zerowy element jej szeregu Fouriera',
                                    show=False)
            ax = wykres.ax
            plt.legend(loc='center right')
            if okres_z_pi:
                ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                ax.set_xticks(list(ax.get_xticks()),
                              ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                               "$-\\frac{1}{2}\pi$",
                               "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                               '$3\pi$']);
            # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_0.png', dpi=300, bbox_inches="tight")
            pdf.savefig(bbox_inches="tight")
            wykres.close()

            wykres = plot_piecewise(f, F_n(1),
                                    xlim=(
                                        (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                            -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                    size=(10, 2.5),
                                    nb_of_points=1000,
                                    dots=True,
                                    legend=True,
                                    label=["$f(x)$", f"$S_1(x)$"],
                                    ylabel='',
                                    title=f'Funkcja $f(x)$ i $S_1(x)$ jej szeregu Fouriera',
                                    show=False)
            ax = wykres.ax
            plt.legend(loc='center right')
            if okres_z_pi:
                ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                ax.set_xticks(list(ax.get_xticks()),
                              ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$", "$-\\frac{1}{2}\pi$",
                               "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                               '$3\pi$']);
            # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_1.png', dpi=300, bbox_inches="tight")
            pdf.savefig(bbox_inches="tight")
            wykres.close()
            #
            for ile_wyrazow in range(2, 20, 1):
                wykres = plot_piecewise(f, F_n(ile_wyrazow),
                                        xlim=(
                                            (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                                -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                        size=(10, 2.5),
                                        nb_of_points=1000 * ile_wyrazow,
                                        dots=True,
                                        legend=True,
                                        label=["$f(x)$", f"$S_{{{ile_wyrazow}}}(x)$"],
                                        ylabel='',
                                        title=f'Funkcja $f(x)$ i $S_{{{ile_wyrazow}}}(x)$  jej szeregu Fouriera',
                                        show=False)
                ax = wykres.ax
                plt.legend(loc='center right')
                if okres_z_pi:
                    ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                    ax.set_xticks(list(ax.get_xticks()),
                                  ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                                   "$-\\frac{1}{2}\pi$",
                                   "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                                   '$3\pi$']);
                # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_{ile_wyrazow}.png', dpi=300, bbox_inches="tight")
                pdf.savefig(bbox_inches="tight")
                wykres.close()
            for ile_wyrazow in range(20, 110, 10):
                wykres = plot_piecewise(f, F_n(ile_wyrazow),
                                        xlim=(
                                            (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                                -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                        size=(10, 2.5),
                                        nb_of_points=1000 * ile_wyrazow,
                                        dots=True,
                                        legend=True,
                                        label=["$f(x)$", f"$S_{{{ile_wyrazow}}}(x)$"],
                                        ylabel='',
                                        title=f'Funkcja $f(x)$ i $S_{{{ile_wyrazow}}}(x)$ jej szeregu Fouriera',
                                        show=False)
                ax = wykres.ax
                plt.legend(loc='center right')
                if okres_z_pi:
                    ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                    ax.set_xticks(list(ax.get_xticks()),
                                  ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                                   "$-\\frac{1}{2}\pi$",
                                   "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                                   '$3\pi$']);
               # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_{ile_wyrazow}.png', dpi=300, bbox_inches="tight")
                pdf.savefig(bbox_inches="tight")
                wykres.close()

            for ile_wyrazow in range(200, 1001, 800):
                wykres = plot_piecewise(f, F_n(ile_wyrazow),
                                        xlim=(
                                            (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                                -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                        size=(10, 2.5),
                                        nb_of_points=1000 * ile_wyrazow,
                                        dots=True,
                                        legend=True,
                                        label=["$f(x)$", f"$S_{{{ile_wyrazow}}}(x)$"],
                                        ylabel='',
                                        title=f'Funkcja $f(x)$ i $S_{{{ile_wyrazow}}}(x)$ jej szeregu Fouriera',
                                        show=False)
                ax = wykres.ax
                plt.legend(loc='center right')
                if okres_z_pi:
                    ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                    ax.set_xticks(list(ax.get_xticks()),
                                  ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                                   "$-\\frac{1}{2}\pi$",
                                   "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                                   '$3\pi$']);
               # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_{ile_wyrazow}.png', dpi=300, bbox_inches="tight")
                pdf.savefig(bbox_inches="tight")

            srednia_T = sp.Rational(1, 2) * (sp.limit(f_left, x, -T, '+') + sp.limit(f_right, x, T, '-'))
            srednia_0 = sp.Rational(1, 2) * (sp.limit(f_left, x, 0, '-') + sp.limit(f_right, x, 0, '+'))
            funkcja = [[(f_left.subs(x, x - 2 * i * T), x < (2 * i) * T), (srednia_0, sp.Eq(x, (2 * i) * T)),
                        (f_right.subs(x, x - 2 * i * T), x < (2 * i + 1) * T), (srednia_T, sp.Eq(x, (2 * i + 1) * T))] for
                       i in range(-9, 10, 1)]
            f = sp.Piecewise(*sum(funkcja, []))
            wykres = plot_piecewise(f, {"color": "C1"},
                                    xlim=(
                                        (max(-10, -4.4 * T), min(4.4 * T, 10)) if okres_z_pi is False else (
                                            -3 * sp.pi - 0.2, 3 * sp.pi + 0.2)),
                                    size=(10, 2.5),
                                    nb_of_points=1000,
                                    dots=True,
                                    legend=True,
                                    label=["$S_{{{\\infty}}}(x)$"],
                                    ylabel='',
                                    title=f'Suma szeregu Fouriera funkcji $f(x)$',
                                    show=False)
            ax = wykres.ax
            plt.legend(loc='center right')
            if okres_z_pi:
                ax.set_xticks([0.5 * i * np.pi for i in range(-6, 7)])
                ax.set_xticks(list(ax.get_xticks()),
                              ["$-3\pi$", "$-\\frac{5}{2}\pi$", "$-2\pi$", "$-\\frac{3}{2}\pi$", "$-\pi$",
                               "$-\\frac{1}{2}\pi$",
                               "$0$", "$\\frac{1}{2}\pi$", "$\pi$", '$\\frac{3}{2}\pi$', '$2\pi$', '$\\frac{5}{2}\pi$',
                               '$3\pi$']);
            # wykres.save(f'./pics/Szereg_Fouriera_{nr_zadania}_inf.pdf', dpi=300, bbox_inches="tight")
            pdf.savefig(bbox_inches="tight")

    if lewostronnie_0 and lewostronnie_T:
        funkcja = (f'\t\t\t{sp.latex(f_left)} \\textnormal{{ dla }}x\\in\\left[{sp.latex(-T)},0\\right)\\\\\n'
                   f'\t\t\t{sp.latex(f_right)} \\textnormal{{ dla }}x\\in\\left[0,{sp.latex(T)}\\right)\n')
    if lewostronnie_0 and not lewostronnie_T:
        funkcja = (f'\t\t\t{sp.latex(f_left)} \\textnormal{{ dla }}x\\in\\left({sp.latex(-T)},0\\right)\\\\\n'
                   f'\t\t\t{sp.latex(f_right)} \\textnormal{{ dla }}x\\in\\left[0,{sp.latex(T)}\\right]\n')
    if not lewostronnie_0 and lewostronnie_T:
        funkcja = (f'\t\t\t{sp.latex(f_left)} \\textnormal{{ dla }}x\\in\\left[{sp.latex(-T)},0\\right]\\\\\n'
                   f'\t\t\t{sp.latex(f_right)} \\textnormal{{ dla }}x\\in\\left(0,{sp.latex(T)}\\right)\n')
    if not lewostronnie_0 and not lewostronnie_T:
        funkcja = (f'\t\t\t{sp.latex(f_left)} \\textnormal{{ dla }}x\\in\\left({sp.latex(-T)},0\\right]\\\\\n'
                   f'\t\t\t{sp.latex(f_right)} \\textnormal{{ dla }}x\\in\\left(0,{sp.latex(T)}\\right]\n')

    if bez_wykresu is True:
        return (f'Rozwinąć w szereg Fouriera funkcję o okresie zasadniczym $2T={sp.latex(2*T)}$ \n'
                f'\t\\[\n'
                f'\t\tf(x)=\\begin{{cases}}\n' +
                funkcja +
                f'\t\t\\end{{cases}}\n'
                f'\t\\]\n'
                ' Naszkicować wykres funkcji, do której zbieżny jest uzyskany szereg.',
                f'$a_0={sp.latex(a0)},\\quad a_n={sp.latex(an)},\\quad b_n={sp.latex(bn)},$\\\\\n'
                f'\t$S(x) = {sp.latex(F_n(5)) if an == 0 or bn == 0 else sp.latex(F_n(3))} + \\dots $')
    else:
        return (f'Rozwinąć w szereg Fouriera funkcję o okresie zasadniczym $2T={sp.latex(2*T)}$ \n'
                f'\t\\[\n'
                f'\t\tf(x)=\\begin{{cases}}\n' +
                funkcja +
                f'\t\t\\end{{cases}}\n'
                f'\t\\]\n'
                ' Naszkicować wykres funkcji, do której zbieżny jest uzyskany szereg.',
                f'$a_0={sp.latex(a0)},\\quad a_n={sp.latex(an)},\\quad b_n={sp.latex(bn)},$\\\\\n'
                f'\t$S(x) = {sp.latex(F_n(5)) if an == 0 or bn == 0 else sp.latex(F_n(3))} + \\dots $\\\\\n'
                f'\t\\animategraphics[height=5.3cm,controls=true]{{0.5}}\n'
                f'\t {{szereg_Fouriera_{nr_zadania}}}{{0}}{{100}}')



if __name__ == "__main__":  # to się uruchamia tylko, gdy plik jest uruchamiany jako program, a nie ładowany jako moduł
    start_time = time.time()
    for i in range(1):
        polecenie, rozwiazanie = szereg_Fouriera(typ_l=0, typ_p=0, bez_wykresu=True, nr_zadania=1)
        print(polecenie, '\n', rozwiazanie)
    print("Czas generowania --- %s seconds ---" % (time.time() - start_time))



from importlib.metadata import version
__version__ = version("generator_zadan")

from .zespolone import rownanie_liniowe
from .zespolone import rownanie_kwadratowe
from .zespolone import pierwiastek_zespolony
from .zespolone import rownanie_ze_sprzezeniem
from .zespolone import obszar_zespolony
from .uklady_rownan_liniowych import uklad_rownan_nieoznaczony
from .uklady_rownan_liniowych import uklad_Cramera
from .macierze import wyznacznik_parametr
from .macierze import macierz_odwrotna_parametr  #strasznie wolne - 14s na jeden 3x3, ale było też 85s!!!
from .macierze import rownanie_macierzowe
from .macierze import wartosci_wlasne
from .macierze import diagonalizacja_macierzy
from .macierze import rzad_macierzy
from .macierze import diagonalizacja_macierzy_z_wielokrotnym_wartosciami_wlasnymi
from .geometria_analityczna import rownanie_prostej
from .geometria_analityczna import rownanie_plaszczyzny
from .geometria_analityczna import odleglosc_prostych_skosnych
from .geometria_analityczna import punkt_symetryczny_do_plaszczyzny
from .geometria_analityczna import punkt_symetryczny_do_prostej
from .geometria_analityczna import katy_w_trojkacie
from .geometria_analityczna import pole_trojkata
from .geometria_analityczna import plaszczyzna_styczna
from .granice import granica_ciagu
from .granice import granica_funkcji
from .monotonicznosc import monotonicznosc
from .monotonicznosc import styczna_normalna
from .calki_nieoznaczone import calka_nieoznaczona
from .calki_nieoznaczone import calka_wymierna
from .calki_oznaczone import calka_oznaczona
from .szeregi_Fouriera import szereg_Fouriera
from .calki_podwojne import calka_podwojna
from .generuj_zestaw import generuj_LaTeX
from .generuj_zestaw import dodaj_zadanie



import os  # do tworzenia katalogów
import random
import sys  # do wysyłania komunikatów w czerwonym kolorze
import time

''' Następne linijki likwidują konflikt z innymi modułami 
Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
import matplotlib.pyplot as plt
''' Poprzednie linijki likwidują konflikt z innymi modułami 
Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
import sympy as sp
from spb import plot, plot_parametric
x = sp.Symbol('x', real=True)
y = sp.Symbol('y', real=True)


# from sympy.integrals.intpoly import polytope_integrate, Polygon # do testów - coś źle liczy dla
# A = [-3, -1]
# B = [-1, 0]
# C = [-1, 3]
# z = 3*x-3 * y

def calka_podwojna(typ: int = 1, nr_zadania: int = 1):
    ''' Następne linijki likwidują konflikt z innymi modułami
    Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
    Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
    plt.rc('text', usetex=False)
    ''' Poprzednie linijki likwidują konflikt z innymi modułami 
    Strasznie to głupie - powiązane z plt.rc('text', usetex=False)
    Chodzi o konfikt generowania wykresów używających TeX - matplotlib i problematycznych Sympy Plot'''
    if not os.path.exists('pics'):
        os.makedirs('pics')
        print(" ! Tworzę katalog pics ", file=sys.stderr)


    if typ == 1:
        a, b = [random.choice(
            [-3, -2, -1, 0, sp.Rational(1, 2), 1, sp.Rational(3, 2), 2, 3]) for _ in
            range(2)]
        c = random.choice([-1, 0, 1, 2])
        while True:
            A = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
            B = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
            C = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
            if (B[0] - A[0]) * (C[1] - A[1]) - (B[1] - A[1]) * (C[0] - A[0]):  # pole trójkąta != 0
                break


        # A = [0, 0]  # pamietaj o sortowaniu
        # B = [1, 2]
        # C = [2, 3]
        A_s, B_s, C_s = A, B, C
        A, B, C = sorted([A, B, C])
        # a, b, c = sp.Rational(1,2), -2, 1
        # A[0], A[1] = A[1], A[0]
        # B[0], B[1] = B[1], B[0]
        # C[0], C[1] = C[1], C[0]
        # a,b = b,a
        # a = 3
        # b= -2
        # c = 1
        z = a * x + b * y + c  # do liczenia względem x
        z_y = b * x + a * y + c  # do liczenia względem y
        # z = 3*x -2*y +1
        # print(A, B, C, a, b, c)
        # pol_int = polytope_integrate(Polygon(A, B, C), z)
        # print(pol_int)
        zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"
        # print(A, B, C)
        if A[0] == B[0]:
            # print("pierwszy")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((C[1] - B[1]), (C[0] - B[0])) * (x - B[0]) + B[1]
            if A[1] > B[1]:
                # print("f jest wyżej")
                print("1 - Jak to widzisz to niedobrze. Daj znać autorowi. - Sortowanie!!!")
                wynik_x = sp.integrate(z, (y, g, f), (x, A[0], C[0]))
                calka_x = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"
            else:
                # print("g jest wyżej")
                wynik_x = sp.integrate(z, (y, f, g), (x, A[0], C[0]))
                calka_x = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                           f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left({sp.latex(sp.integrate(z, (y, f, g)).expand())}\\right)dx")
                szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
                wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
                p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                          # markers=[
                          #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                          #     {"args": [A[0], A[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                          #     {"args": [B[0], B[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                          #     {"args": [C[0], C[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                          # ],
                          xlim=(min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc
                                else (min(A[0], B[0], C[0]) + max(A[0], B[0], C[0])) / 2 - wysokosc / 2,
                                max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc
                                else (min(A[0], B[0], C[0]) + max(A[0], B[0], C[0])) / 2 + wysokosc / 2),
                          ylim=((min(A[1], B[1], C[1]) + max(A[1], B[1], C[1])) / 2 - szerokosc / 2 if
                                szerokosc > wysokosc else
                                min(A[1], B[1], C[1]) - 1,
                                (min(A[1], B[1], C[1]) + max(A[1], B[1], C[1])) / 2 + szerokosc / 2
                                if szerokosc > wysokosc else max(A[1], B[1], C[1]) + 1),
                          aspect=(1, 1),
                          label=f'$y={sp.latex(f)}$',
                          show=False,
                          title=f'Obszar całkowania względem osi $Ox$',
                          xlabel="x",
                          ylabel="y",
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )

                p2 = plot(g,  # plot a symbolic expression to force to use 2D plot
                          markers=[
                              {"args": [[A[0], A[0]], [-10, 10]], 'label': f'$x={A[0]}$'},  # prosta pionowa
                              {"args": [A_s[0], A_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[0],A_s[1]}'},
                              {"args": [B_s[0], B_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[0],B_s[1]}'},
                              {"args": [C_s[0], C_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[0],C_s[1]}'},
                          ],
                          label=f'$y={sp.latex(g)}$',
                          show=False,
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p1.append(p2[0])
                p1.append(p2[1])
                p1.append(p2[2])
                p1.append(p2[3])
                p1.append(p2[4])
                # p1.show()
                p1.save(f'./pics/calka_podwojna{nr_zadania}x.pdf')
                p1.save(f'./pics/calka_podwojna{nr_zadania}x.png')
                p1.close()
                p2.close()

            # if wynik_x == pol_int:
            #     print("SUKCES")
            # return (
            #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$  dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
            #     '$' + calka_x + ' = ' + sp.latex(wynik_x) + '$')
        elif B[0] == C[0]:
            # print("drugi")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((A[1] - B[1]), (A[0] - B[0])) * (x - B[0]) + B[1]
            if B[1] > C[1]:
                # print("B jest wyżej")
                print("2 - Jak to widzisz to niedobrze. Daj znać autorowi. - Sortowanie!!!")
                wynik_x = sp.integrate(z, (y, f, g), (x, A[0], C[0]))
                calka_x = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"
            else:
                # print("C jest wyżej")
                wynik_x = sp.integrate(z, (y, g, f), (x, A[0], C[0]))
                calka_x = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                           f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left({sp.latex(sp.integrate(z, (y, g, f)).expand())}\\right)dx")

                szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
                wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
                p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                          # markers=[
                          #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                          #     {"args": [A[0], A[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                          #     {"args": [B[0], B[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                          #     {"args": [C[0], C[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                          # ],
                          xlim=(min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 - wysokosc / 2,
                                max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 + wysokosc / 2),
                          ylim=((min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 - szerokosc / 2 if szerokosc > wysokosc else min(
                              A[1], B[1], C[1]) - 1,
                                (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 + szerokosc / 2 if szerokosc > wysokosc else max(
                                    A[1], B[1], C[1]) + 1),
                          aspect=(1, 1),
                          label=f'$y={sp.latex(f)}$',
                          show=False,
                          title=f'Obszar całkowania względem osi $Ox$',
                          xlabel="x",
                          ylabel="y",
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p2 = plot(g,  # plot a symbolic expression to force to use 2D plot
                          markers=[
                              {"args": [[B[0], B[0]], [-10, 10]], 'label': f'$x={B[0]}$'},  # prosta pionowa
                              {"args": [A_s[0], A_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[0],A_s[1]}'},
                              {"args": [B_s[0], B_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[0],B_s[1]}'},
                              {"args": [C_s[0], C_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[0],C_s[1]}'},
                          ],
                          label=f'$y={sp.latex(g)}$',
                          show=False,
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p1.append(p2[0])
                p1.append(p2[1])
                p1.append(p2[2])
                p1.append(p2[3])
                p1.append(p2[4])
                # p1.show()
                p1.save(f'./pics/calka_podwojna{nr_zadania}x.pdf')
                p1.save(f'./pics/calka_podwojna{nr_zadania}x.png')
                p1.close()
                p2.close()

            # if wynik_x == pol_int:
            #     print("SUKCES")
            # return (
            #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
            #     '$' + calka_x + ' = ' + sp.latex(wynik_x) + '$')
        else:
            # print("trzeci")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((C[1] - B[1]), (C[0] - B[0])) * (x - B[0]) + B[1]
            h = sp.Rational((A[1] - B[1]), (A[0] - B[0])) * (x - B[0]) + B[1]
            # print("f = ", f)
            # print("g = ", g)
            # print("h = ", h)
            if A[1] + (B[0] - A[0]) / (C[0] - A[0]) * (C[1] - A[1]) > B[1]:
                # print("środkowy poniżej")
                wynik_x = sp.integrate(z, (y, h, f), (x, A[0], B[0])) + sp.integrate(z, (y, g, f), (x, B[0], C[0]))
                calka_x = (
                        f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(h)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                        f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left( {sp.latex(sp.integrate(z, (y, h, f)).expand())}   \\right)dx + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(  {sp.latex(sp.integrate(z, (y, g, f)).expand())}    \\right)dx"
                )
            else:
                # print("środkowy powyżej")
                wynik_x = sp.integrate(z, (y, f, h), (x, A[0], B[0])) + sp.integrate(z, (y, f, g), (x, B[0], C[0]))
                calka_x = (
                        f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(h)}}}\\left({sp.latex(z)}\\right)dy\\right)dx + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                        f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left( {sp.latex(sp.integrate(z, (y, f, h)).expand())}   \\right)dx + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(  {sp.latex(sp.integrate(z, (y, f, g)).expand())}    \\right)dx"
                )
            szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
            wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
            p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                      # markers=[
                      #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                      #     {"args": [A[0], A[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                      #     {"args": [B[0], B[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                      #     {"args": [C[0], C[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                      # ],
                      xlim=(min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0], B[0],
                                                                                                                C[
                                                                                                                    0])) / 2 - wysokosc / 2,
                            max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0], B[0],
                                                                                                                C[
                                                                                                                    0])) / 2 + wysokosc / 2),
                      ylim=(
                          (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                       C[1])) / 2 - szerokosc / 2 if szerokosc > wysokosc else min(
                              A[1], B[1], C[1]) - 1,
                          (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                       C[1])) / 2 + szerokosc / 2 if szerokosc > wysokosc else max(
                              A[1], B[1], C[1]) + 1),
                      aspect=(1, 1),
                      label=f'$y={sp.latex(f)}$',
                      show=False,
                      title='Obszar całkowania względem osi $Ox$',
                      xlabel="x",
                      ylabel="y",
                      # use_latex=True,
                      size=(4.5, 4.5),

                      )
            p2 = plot(h,  # plot a symbolic expression to force to use 2D plot
                      label=f'$y={sp.latex(h)}$',
                      show=False,
                      size=(4.5, 4.5),

                      # use_latex=True
                      )
            p3 = plot(g,  # plot a symbolic expression to force to use 2D plot
                      markers=[
                          {"args": [A_s[0], A_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[0],A_s[1]}'},
                          {"args": [B_s[0], B_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[0],B_s[1]}'},
                          {"args": [C_s[0], C_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[0],C_s[1]}'},
                      ],
                      label=f'$y={sp.latex(g)}$',
                      # use_latex=True,
                      size=(4.5, 4.5),

                      show=False,

                      )
            p1.append(p2[0])
            p1.append(p3[0])
            p1.append(p3[1])
            p1.append(p3[2])
            p1.append(p3[3])
            # p1.show()
            p1.save(f'./pics/calka_podwojna{nr_zadania}x.pdf')
            p1.save(f'./pics/calka_podwojna{nr_zadania}x.png')
            p1.close()
            p2.close()
            p3.close()
            # if wynik_x == pol_int:
            #     print("SUKCES")
        ''' zamieniam x i y miejscami - raczej można to zrobić lepiej '''
        A[0], A[1] = A[1], A[0]
        B[0], B[1] = B[1], B[0]
        C[0], C[1] = C[1], C[0]
        A, B, C = sorted([A, B, C])

        # print(A, B, C)
        # pol_int = polytope_integrate(Polygon(A, B, C), z)
        # print(pol_int)
        # zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"
        if A[0] == B[0]:
            # print("czwarty")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((C[1] - B[1]), (C[0] - B[0])) * (x - B[0]) + B[1]
            if A[1] > B[1]:
                # print("f jest wyżej")
                print("4 - Jak to widzisz to niedobrze. Daj znać autorowi. - Sortowanie!!!")
                wynik_y = sp.integrate(z_y, (y, g, f), (x, A[0], C[0]))
                calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x', 'y')}}}^{{{sp.latex(f).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
            else:
                # print("g jest wyżej")
                wynik_y = sp.integrate(z_y, (y, f, g), (x, A[0], C[0]))
                calka_y = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x', 'y')}}}^{{{sp.latex(g).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"+
                           f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left({sp.latex(sp.integrate(z_y, (y, f, g)).expand()).replace('x', 'y')}\\right)dy")

                szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
                wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
                p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                          # markers=[
                          #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                          #     {"args": [A[0], A[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                          #     {"args": [B[0], B[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                          #     {"args": [C[0], C[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                          # ],
                          xlim=(max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 + wysokosc / 2,
                                min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 - wysokosc / 2),
                          ylim=((min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 - szerokosc / 2 if szerokosc > wysokosc else min(
                              A[1], B[1], C[1]) - 1,
                                (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 + szerokosc / 2 if szerokosc > wysokosc else max(
                                    A[1], B[1], C[1]) + 1),
                          aspect=(1, 1),
                          label='$x = $' + f'${sp.latex(f)}$'.replace('x', 'y'),
                          show=False,
                          title=f'Obszar całkowania względem osi $Oy$',
                          xlabel="y",
                          ylabel="x",
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p2 = plot(g,  # plot a symbolic expression to force to use 2D plot
                          markers=[
                              {"args": [[A[0], A[0]], [-10, 10]], 'label': f'$y={A[0]}$'},  # prosta pionowa
                              {"args": [A_s[0], A_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[1],A_s[0]}'},
                              {"args": [B_s[0], B_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[1],B_s[0]}'},
                              {"args": [C_s[0], C_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[1],C_s[0]}'},
                          ],
                          label='$x = $' + f'${sp.latex(g)}$'.replace('x', 'y'),
                          show=False,
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p1.append(p2[0])
                p1.append(p2[1])
                p1.append(p2[2])
                p1.append(p2[3])
                p1.append(p2[4])
                # p1.show()
                p1.save(f'./pics/calka_podwojna{nr_zadania}y.pdf')
                p1.save(f'./pics/calka_podwojna{nr_zadania}y.png')
                p1.close()
                p2.close()

            # if wynik_y == pol_int:
            #     print("SUKCES")
            # return (
            #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$  dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
            #     '$' + calka_y + ' = ' + sp.latex(wynik_y) + '$')
        elif B[0] == C[0]:
            # print("piąty")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((A[1] - B[1]), (A[0] - B[0])) * (x - B[0]) + B[1]
            if B[1] > C[1]:
                # print("B jest wyżej")
                print("5 - Jak to widzisz to niedobrze. Daj znać autorowi. - Sortowanie!!!")
                wynik_y = sp.integrate(z_y, (y, f, g), (x, A[0], C[0]))
                calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x', 'y')}}}^{{{sp.latex(g).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
            else:
                # print("C jest wyżej")
                wynik_y = sp.integrate(z_y, (y, g, f), (x, A[0], C[0]))
                calka_y = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x', 'y')}}}^{{{sp.latex(f).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"+
                           f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left({sp.latex(sp.integrate(z_y, (y, g, f)).expand()).replace('x', 'y')}\\right)dy")

                szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
                wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
                p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                          # markers=[
                          #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                          #     {"args": [A[0], A[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                          #     {"args": [B[0], B[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                          #     {"args": [C[0], C[1]], "marker": "o",
                          #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                          # ],
                          xlim=(max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 + wysokosc / 2,
                                min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0],
                                                                                                                    B[0], C[
                                                                                                                        0])) / 2 - wysokosc / 2),
                          ylim=((min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 - szerokosc / 2 if szerokosc > wysokosc else min(
                              A[1], B[1], C[1]) - 1,
                                (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                             C[1])) / 2 + szerokosc / 2 if szerokosc > wysokosc else max(
                                    A[1], B[1], C[1]) + 1),
                          aspect=(1, 1),
                          label='$x = $' + f'${sp.latex(f)}$'.replace('x', 'y'),
                          show=False,
                          title=f'Obszar całkowania względem osi $Oy$',
                          xlabel="y",
                          ylabel="x",
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p2 = plot(g,  # plot a symbolic expression to force to use 2D plot
                          markers=[
                              {"args": [[B[0], B[0]], [-10, 10]], 'label': f'$y={B[0]}$'},  # prosta pionowa
                              {"args": [A_s[0], A_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[1],A_s[0]}'},
                              {"args": [B_s[0], B_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[1],B_s[0]}'},
                              {"args": [C_s[0], C_s[1]], "marker": "o",
                               "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[1],C_s[0]}'},
                          ],
                          label='$x = $' + f'${sp.latex(g)}$'.replace('x', 'y'),
                          show=False,
                          # use_latex=True,
                          size=(4.5, 4.5),

                          )
                p1.append(p2[0])
                p1.append(p2[1])
                p1.append(p2[2])
                p1.append(p2[3])
                p1.append(p2[4])
                # p1.show()
                p1.save(f'./pics/calka_podwojna{nr_zadania}y.pdf')
                p1.save(f'./pics/calka_podwojna{nr_zadania}y.png')
                p1.close()
                p2.close()
            # if wynik_y == pol_int:
            #     print("SUKCES")
            # return (
            #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
            #     '$' + calka_y + ' = ' + sp.latex(wynik_y) + '$')
        else:
            # print("szósty")
            f = sp.Rational((C[1] - A[1]), (C[0] - A[0])) * (x - A[0]) + A[1]
            g = sp.Rational((C[1] - B[1]), (C[0] - B[0])) * (x - B[0]) + B[1]
            h = sp.Rational((A[1] - B[1]), (A[0] - B[0])) * (x - B[0]) + B[1]
            # print("f = ", f)
            # print("g = ", g)
            # print("h = ", h)
            if A[1] + (B[0] - A[0]) / (C[0] - A[0]) * (C[1] - A[1]) > B[1]:
                # print("środkowy poniżej")
                wynik_y = sp.integrate(z_y, (y, h, f), (x, A[0], B[0])) + sp.integrate(z_y, (y, g, f), (x, B[0], C[0]))
                calka_y = (
                        f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(h).replace('x', 'y')}}}^{{{sp.latex(f).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x', 'y')}}}^{{{sp.latex(f).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"+
                        f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(  {sp.latex(sp.integrate(z_y, (y, h, f)).expand()).replace('x', 'y')}  \\right)dy + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(  {sp.latex(sp.integrate(z_y, (y, g, f)).expand()).replace('x', 'y')}  \\right)dy")
            else:
                # print("środkowy powyżej")
                wynik_y = sp.integrate(z_y, (y, f, h), (x, A[0], B[0])) + sp.integrate(z_y, (y, f, g), (x, B[0], C[0]))
                calka_y = (
                        f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x', 'y')}}}^{{{sp.latex(h).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x', 'y')}}}^{{{sp.latex(g).replace('x', 'y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"+
                        f"\n\t=\ldots=\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(  {sp.latex(sp.integrate(z_y, (y, f, h)).expand()).replace('x', 'y')}  \\right)dy + " +
                        f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(  {sp.latex(sp.integrate(z_y, (y, f, g)).expand()).replace('x', 'y')}  \\right)dy")
            # if wynik_y == pol_int:
            #     print("SUKCES")
            szerokosc = max(A[0], B[0], C[0]) + 1 - (min(A[0], B[0], C[0]) - 1)
            wysokosc = max(A[1], B[1], C[1]) + 1 - (min(A[1], B[1], C[1]) - 1)
            p1 = plot(f,  # plot a symbolic expression to force to use 2D plot
                      # markers=[
                      #     {"args": [[A[0], A[0]], [-10, 10]], 'label': f'x={A[0]}'},  # prosta pionowa
                      #     {"args": [A[0], A[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'A'},
                      #     {"args": [B[0], B[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'B'},
                      #     {"args": [C[0], C[1]], "marker": "o",
                      #      "linestyle": "None", 'markersize': '10', 'label': 'C'},
                      # ],
                      xlim=(max(A[0], B[0], C[0]) + 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0], B[0],
                                                                                                                C[
                                                                                                                    0])) / 2 + wysokosc / 2,
                            min(A[0], B[0], C[0]) - 1 if szerokosc > wysokosc else (min(A[0], B[0], C[0]) + max(A[0], B[0],
                                                                                                                C[
                                                                                                                    0])) / 2 - wysokosc / 2),
                      ylim=(
                          (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                       C[1])) / 2 - szerokosc / 2 if szerokosc > wysokosc else min(
                              A[1], B[1], C[1]) - 1,
                          (min(A[1], B[1], C[1]) + max(A[1], B[1],
                                                       C[1])) / 2 + szerokosc / 2 if szerokosc > wysokosc else max(
                              A[1], B[1], C[1]) + 1),
                      aspect=(1, 1),
                      label='$x = $' + f'${sp.latex(f)}$'.replace('x', 'y'),
                      show=False,
                      title='Obszar całkowania względem osi $Oy$',
                      xlabel="y",
                      ylabel="x",
                      # use_latex=True
                      size=(4.5, 4.5),

                      )
            p2 = plot(h,  # plot a symbolic expression to force to use 2D plot
                      label='$x = $' + f'${sp.latex(h)}$'.replace('x', 'y'),
                      show=False,
                      size=(4.5, 4.5),

                      # use_latex=True
                      )
            p3 = plot(g,  # plot a symbolic expression to force to use 2D plot
                      markers=[
                          {"args": [A_s[0], A_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'A={A_s[1],A_s[0]}'},
                          {"args": [B_s[0], B_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'B={B_s[1],B_s[0]}'},
                          {"args": [C_s[0], C_s[1]], "marker": "o",
                           "linestyle": "None", 'markersize': '10', 'label': f'C={C_s[1],C_s[0]}'},
                      ],
                      label='$x = $' + f'${sp.latex(g)}$'.replace('x', 'y'),
                      # use_latex=True,
                      size=(4.5, 4.5),

                      show=False,
                      )
            p1.append(p2[0])
            p1.append(p3[0])
            p1.append(p3[1])
            p1.append(p3[2])
            p1.append(p3[3])
            # p1.show()
            p1.save(f'./pics/calka_podwojna{nr_zadania}y.pdf')
            p1.save(f'./pics/calka_podwojna{nr_zadania}y.png')
            p1.close()
            p2.close()
            p3.close()
        A[0], A[1] = A[1], A[0]
        B[0], B[1] = B[1], B[0]
        C[0], C[1] = C[1], C[0]
        # wynik_y = 1
        if wynik_y != wynik_x:
            print('\n \x1b[1;7;31m' + '!!!Różne wyniki względem osi!!!  -  Posortowałeś punkty ABC?' + '\x1b[0m \n ')
        return (
            f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A_s[0]},{A_s[1]}), B=({B_s[0]},{B_s[1]}), C=({C_s[0]},{C_s[1]})$',
            f'\\includegraphics[width=60mm]{{../pics/calka_podwojna{nr_zadania}x}}\n' +
            f'\t\\includegraphics[width=60mm]{{../pics/calka_podwojna{nr_zadania}y}}\\\\\n' +
            '\tWzględem $Ox:$ $' + calka_x + ' = \ldots = ' + sp.latex(wynik_x) + '$ \\\\\n' +
            '\tWzględem $Oy:$ $' + calka_y + ' = \ldots = ' + sp.latex(wynik_y) + '$ \\\\')
    if typ == 2:
        wzgledem_xy = random.choice(['x','y'])
        # wzgledem_xy = 'y'
        if wzgledem_xy == 'x':
            while True:
                a, d = [
                    random.choice([-3, -2, -1, sp.Rational(1, 2), 1, sp.Rational(3, 2), 2, 3])
                    for _ in range(2)]
                b, c, e = [
                    random.choice([-3, -2, -1, sp.Rational(1, 2), 0, 0, 0, 0, 0, 1, sp.Rational(3, 2), 2, 3])
                    for _ in range(3)]
                # a = -3 # sp.Rational(1, 2)
                # b = -1
                # c = 3
                # d = -1
                # e = 0
                # a, b, c = -1, 3, -3 #, -1/2, -3
                # d, e = sp.Rational(1, 2), -3
                # print(a, b, c, d, e)
                f = a * x ** 2 + b * x + c
                g = d * x + e

                roz = sp.solve(f - g)
                # print((roz[0].is_Rational),type(roz[1]))
                # print(roz)

                if len(roz) == 2 and all([roz[i].is_integer for i in range(2)]):
                    break
            while True:
                A, B = [random.choice(
                    [-3, -2, -1, sp.Rational(1, 2), 1, 0, 0, 0, sp.Rational(3, 2), 2, 3]) for _ in
                    range(2)]
                if A**2 + B**2 > 0:
                    break

            C = random.choice([-1, 0, 0, 0, 1, 2])
            z = A * x + B * y + C
            # print(f,g,roz,sep="\n")
            szerokosc = max(roz) - min(roz)
            wysokosc = max(f.subs(x, roz[0]), f.subs(x, roz[1])) + (b ** 2 - 4 * a * c) / (4 * a) if a > 0 else \
                - (b ** 2 - 4 * a * c) / (4 * a) - (min(f.subs(x, roz[0]), f.subs(x, roz[1])))
            y_min = - (b ** 2 - 4 * a * c) / (4 * a) if a > 0 else min(f.subs(x, roz[0]), f.subs(x, roz[1]))
            y_max = max(f.subs(x, roz[0]), f.subs(x, roz[1])) if a > 0 else - (b ** 2 - 4 * a * c) / (4 * a)
            p = plot(f,
                     g,
                     (x, -100, 100),  # inaczej obcina dziedzinę funkcji do -10,10
                     n=50000,  # powiązane w linijką wyżej. Inaczej wykres nie jest gładki.

                     markers=[
                         {"args": [roz[0], f.subs(x, roz[0])], "marker": "o",
                          "linestyle": "None", 'markersize': '10', 'label': f'A={roz[0], f.subs(x, roz[0])}'},
                         {"args": [roz[1], f.subs(x, roz[1])], "marker": "o",
                          "linestyle": "None", 'markersize': '10', 'label': f'B={roz[1], f.subs(x, roz[1])}'},
                         {"args": [ -b / (2 * a), f.subs(x, -b / (2 * a))], "marker": "o",
                          "linestyle": "None", 'markersize': '10',
                          'label': f'W={sp.Rational(-b, 2 * a), (f.subs(x, sp.Rational(-b, 2 * a)))}'},
                     ],
                     xlim=(min(roz) - 1 if szerokosc > wysokosc else (min(roz) + max(roz)) / 2 - wysokosc / 2 - 1,
                           max(roz) + 1 if szerokosc > wysokosc else (min(roz) + max(roz)) / 2 + wysokosc / 2 + 1),
                     ylim=((y_min + y_max) / 2 - szerokosc / 2 - 1 if szerokosc > wysokosc else y_min - 1,
                           (y_min + y_max) / 2 + szerokosc / 2 + 1 if szerokosc > wysokosc else y_max + 1),
                     # aspect=(1, 1),
                     title='Obszar całkowania względem $Ox$',
                     # title = f"{round(szerokosc), round(wysokosc), round(roz[0]), round (roz[1]), round(y_min), round( y_max)}",
                     label=(f'y=${sp.latex(f)}$', f'y=${sp.latex(g)}$'),

                     show=False,
                     # use_latex=True,
                     size=(4.5, 4.5),

                     xlabel="x",
                     ylabel="y",
                     aspect=(1, 1),
                     )

            p.save(f'./pics/calka_podwojna{nr_zadania}x.pdf')
            p.save(f'./pics/calka_podwojna{nr_zadania}x.png')
            p.close()

            f_i = sp.solve(y - f, x)
            g_i = sp.solve(y - g, x)

            # print((y_min + y_max) / 2 - szerokosc / 2 + 1 if szerokosc > wysokosc else y_min - 1,
            #       (y_min + y_max) / 2 + szerokosc / 2 - 1 if szerokosc > wysokosc else y_max + 1)
            p = plot(f_i[0], f_i[1], g_i,
                 (y, -100, 100),  # inaczej obcina dziedzinę funkcji do -10,10
                 markers=[
                     {"args": [f.subs(x, roz[0]), roz[0]], "marker": "o",
                      "linestyle": "None", 'markersize': '10', 'label': f'A={roz[0], f.subs(x, roz[0])}'},
                     {"args": [f.subs(x, roz[1]), roz[1]], "marker": "o",
                      "linestyle": "None", 'markersize': '10', 'label': f'B={roz[1], f.subs(x, roz[1])}'},
                     {"args": [f.subs(x, -b / (2 * a)), -b / (2 * a)], "marker": "o",
                      "linestyle": "None", 'markersize': '10',
                      'label': f'W={sp.Rational(-b, 2 * a), (f.subs(x, sp.Rational(-b, 2 * a)))}'},
                 ],

                 xlim=((y_min + y_max) / 2 + szerokosc / 2 + 1 if szerokosc > wysokosc else y_max + 1,
                       (y_min + y_max) / 2 - szerokosc / 2 - 1 if szerokosc > wysokosc else y_min - 1),
                 ylim=(min(roz) - 1 if szerokosc > wysokosc else (min(roz) + max(roz)) / 2 - wysokosc / 2 - 1,
                       max(roz) + 1 if szerokosc > wysokosc else (min(roz) + max(roz)) / 2 + wysokosc / 2 + 1),

                 label=(f'x=${sp.latex(f_i[0])}$', f'x=${sp.latex(f_i[1])}$', f'x=${sp.latex(g_i[0])}$'),
                 n=50000,  # dużo by nie było przerwy przy wierzchołku
                 title=f'Obszar całkowania względem osi Oy',
                 xlabel="y",
                 ylabel="x",
                 aspect=(1, 1),
                 show=False,
                 # use_latex=True,
                 size=(4.5, 4.5),
                 )
            p.save(f'./pics/calka_podwojna{nr_zadania}y.pdf')
            p.save(f'./pics/calka_podwojna{nr_zadania}y.png')
            p.close()
            # print(sp.Rational(-b, (2 * a)), (f.subs(x, -b / (2 * a))))




            wynik_x = sp.integrate(z, (y, f, g), (x, min(roz), max(roz))) * (1 if a > 0 else -1)
            calka_x = (
                    f"\\int\\limits_{{{min(roz)}}}^{{{max(roz)}}}\\left(\\int\\limits_{{{sp.latex(f) if a > 0 else sp.latex(g)}}}^{{{sp.latex(g) if a > 0 else sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx" +
                    f"\n\t=\ldots=\\int\\limits_{{{min(roz)}}}^{{{max(roz)}}}\\left({sp.latex(sp.integrate(z, (y, f, g)).expand()) if a > 0 else sp.latex(sp.integrate(z, (y, g, f)).expand())}\\right)dx")

            zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"

            if a > 0 and d > 0:
                if min(roz) < - b / (2 * a) < max(roz):
                    wynik_y = (sp.integrate(z, (x, f_i[0], f_i[1]),
                                            (y, f.subs(x, sp.Rational(-b, (2 * a))), f.subs(x, min(roz)))) +
                               sp.integrate(z, (x, g_i[0], f_i[1]), (y, f.subs(x, min(roz)), f.subs(x, max(roz)))))
                    calka_y = (
                            f"\\int\\limits_{{{f.subs(x, sp.Rational(-b , (2 * a)))}}}^{{{f.subs(x, min(roz))}}}\\left(\\int\\limits_{{{sp.latex(f_i[0])}}}^{{{sp.latex(f_i[1])}}}\\left({sp.latex(z)}\\right)dx\\right)dy + " +
                            f"\n\t\t\\int\\limits_{{{f.subs(x, min(roz))}}}^{{{f.subs(x, max(roz))} }}\\left(\\int\\limits_{{{sp.latex(g_i[0])}}}^{{{sp.latex(f_i[1])}}}\\left({sp.latex(z)}\\right)dx\\right)dy" +
                            f"\n\t=\ldots=\\int\\limits_{{{f.subs(x, sp.Rational(-b , (2 * a)))}}}^{{{f.subs(x, min(roz))}}}\\left({sp.latex(sp.integrate(z, (x, f_i[0], f_i[1])).expand())}\\right)dy +" +
                            f"\n\t\t\\int\\limits_{{{f.subs(x, min(roz))}}}^{{{f.subs(x, max(roz))}}}\\left({sp.latex(sp.integrate(z, (x, g_i[0], f_i[1])).expand())}\\right)dx")

                else:
                    wynik_y = sp.integrate(z, (x, g_i[0], f_i[1]), (y, f.subs(x, min(roz)), f.subs(x, max(roz))))
                    calka_y = (
                            f"\\int\\limits_{{{f.subs(x, min(roz))}}}^{{{f.subs(x, max(roz))}}}\\left(\\int\\limits_{{{sp.latex(g_i[0])}}}^{{{sp.latex(f_i[1])}}}\\left({sp.latex(z)}\\right)dx\\right)dy" +
                            f"\n\t=\ldots=\\int\\limits_{{{f.subs(x, min(roz))}}}^{{{f.subs(x, max(roz))}}}\\left({sp.latex(sp.integrate(z, (x, g_i[0], f_i[1])).expand())}\\right)dy")
                return (
                    f'Obliczyć ${zadanie}$ gdzie $D$ - obszar ograniczony krzywymi $y={sp.latex(f)}$, oraz $y={sp.latex(g)}$',
                    f'\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}x}}\n' +
                    f'\t\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}y}}\\\\\n' +
                    '\tWzględem $Ox:$ $' + calka_x + ' = \ldots = ' + sp.latex(wynik_x) + '$ \\\\\n' +
                    '\tWzględem $Oy:$ $' + calka_y + ' = \ldots = ' + sp.latex(wynik_y) + '$ \\\\')


            else:
                return (
                    f'Obliczyć ${zadanie}$ gdzie $D$ - obszar ograniczony krzywymi $y={sp.latex(f)}$, oraz $y={sp.latex(g)}$',
                    f'\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}x}}\n' +
                    f'\t\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}y}}\\\\\n' +
                    '\tWzględem $Ox:$ $' + calka_x + ' = \ldots = ' + sp.latex(wynik_x) + '$ \\\\')


        else:
            while True:
                a, d = [
                    random.choice([-3, -2, -1, sp.Rational(1, 2), 1, sp.Rational(3, 2), 2, 3])
                    for _ in range(2)]
                b, c, e = [
                    random.choice([-3, -2, -1, sp.Rational(1, 2), 0, 0, 0, 0, 1, sp.Rational(3, 2), 2, 3])
                    for _ in range(3)]

                # a, b, c = -1, 3, -3 #, -1/2, -3
                # d, e = sp.Rational(1, 2), -3
                # print(a, b, c, d, e)
                f = a * y ** 2 + b * y + c
                g = d * y + e
                roz = sp.solve(f - g)
                # print((roz[0].is_Rational),type(roz[1]))
                # print(roz)

                if len(roz) == 2 and all([roz[i].is_integer for i in range(2)]):
                    break
            while True:
                A, B = [random.choice(
                    [-3, -2, -1, sp.Rational(1, 2), 1, 0, 0, 0, sp.Rational(3, 2), 2, 3]) for _ in
                    range(2)]
                if A**2 + B**2 > 0:
                    break
            C = random.choice([-1, 0, 0, 0, 1, 2])
            z = A * x + B * y + C

            x_min = - (b ** 2 - 4 * a * c) / (4 * a) if a > 0 else min(f.subs(y, roz[0]), f.subs(y, roz[1]))
            x_max = max(f.subs(y, roz[0]), f.subs(y, roz[1])) if a > 0 else - (b ** 2 - 4 * a * c) / (4 * a)
            szerokosc = max(f.subs(y, roz[0]), f.subs(y, roz[1])) + (b ** 2 - 4 * a * c) / (4 * a) if a > 0 else \
                - (b ** 2 - 4 * a * c) / (4 * a) - (min(f.subs(y, roz[0]), f.subs(y, roz[1])))

            wysokosc = max(roz) - min(roz)
            # print(x_min, x_max)
            # print(szerokosc, wysokosc)
            f_i = sp.solve(x - f, y)
            g_i = sp.solve(x - g, y)
            p = plot_parametric((f_i[0], x), (f_i[1], x), (g_i, x),
                            (x, -100, 100),
                            n=50000,
                            xlabel="x",
                            ylabel="y",
                            use_cm=False,
                            label=(f'y=${sp.latex(f_i[0])}$', f'y=${sp.latex(f_i[1])}$', f'y=${sp.latex(g_i[0])}$'),
                            markers=[
                                {"args": [roz[0], f.subs(y, roz[0])], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'A={f.subs(y, roz[0]), roz[0]}'},
                                {"args": [roz[1], f.subs(y, roz[1])], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'B={f.subs(y, roz[1]), roz[1]}'},
                                {"args": [-b / (2 * a), f.subs(y, -b / (2 * a))], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'W={(f.subs(y, sp.Rational(-b, 2 * a))), sp.Rational(-b, 2 * a)}'},
                            ],
                            xlim=(
                                max(roz) + 1 if szerokosc > wysokosc else (min(roz) + max(
                                    roz)) / 2 + wysokosc / 2 + 1,
                                min(roz) - 1 if szerokosc > wysokosc else (min(roz) + max(
                                    roz)) / 2 - wysokosc / 2 - 1),
                            ylim=((x_min + x_max) / 2 - szerokosc / 2 - 1 if szerokosc > wysokosc else x_min - 1,
                                  (x_min + x_max) / 2 + szerokosc / 2 + 1 if szerokosc > wysokosc else x_max + 1),

                            # aspect=(1, 1),
                            title=f'Obszar całkowania względem osi Ox',
                            show=False,
                            # use_latex=True,
                            size=(4.5, 4.5),
                            )
            p.save(f'./pics/calka_podwojna{nr_zadania}x.pdf')
            p.save(f'./pics/calka_podwojna{nr_zadania}x.png')
            p.close()


            y_min = x_min
            y_max = x_max
            # print((y_min + y_max) / 2 - szerokosc / 2 + 1 if szerokosc > wysokosc else y_min - 1,
            #       (y_min + y_max) / 2 + szerokosc / 2 - 1 if szerokosc > wysokosc else y_max + 1)
            p = plot_parametric((f, y), (g, y),
                            (y, -100, 100),  # inaczej obcina dziedzinę funkcji do -10,10
                            use_cm=False,
                            markers=[
                                {"args": [f.subs(y, roz[0]), roz[0]], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'A={f.subs(y, roz[0]), roz[0]}'},
                                {"args": [f.subs(y, roz[1]), roz[1]], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'B={f.subs(y, roz[1]), roz[1]}'},
                                {"args": [f.subs(y, -b / (2 * a)), -b / (2 * a)], "marker": "o",
                                 "linestyle": "None", 'markersize': '10',
                                 'label': f'W={(f.subs(y, sp.Rational(-b, 2 * a))), sp.Rational(-b, 2 * a)}'},
                            ],
                            xlim=((x_min + x_max) / 2 - szerokosc / 2 - 1 if szerokosc > wysokosc else x_min - 1,
                                  (x_min + x_max) / 2 + szerokosc / 2 + 1 if szerokosc > wysokosc else x_max + 1),
                            ylim=(
                                min(roz) - 1 if szerokosc > wysokosc else (min(roz) + max(
                                    roz)) / 2 - wysokosc / 2 - 1,
                                max(roz) + 1 if szerokosc > wysokosc else (min(roz) + max(
                                    roz)) / 2 + wysokosc / 2 + 1),
                            label=(f'x=${sp.latex(f)}$', f'x=${sp.latex(g)}$'),

                            n=100000,  # dużo by nie było przerwy przy wierzchołku
                            title=f'Obszar całkowania względem osi Oy',
                            xlabel="y",
                            ylabel="x",
                            # aspect=(1, 1),
                            show=False,
                            # use_latex=True,
                            size=(4.5, 4.5),
                            )
            # print(sp.Rational(-b, 2 * a),f.subs(y, sp.Rational(-b, 2 * a)))
            p.save(f'./pics/calka_podwojna{nr_zadania}y.pdf')
            p.save(f'./pics/calka_podwojna{nr_zadania}y.png')
            p.close()

            wynik_y = sp.integrate(z, (x, f, g), (y, min(roz), max(roz))) * (1 if a > 0 else -1)
            calka_y = (
                    f"\\int\\limits_{{{min(roz)}}}^{{{max(roz)}}}\\left(\\int\\limits_{{{sp.latex(f) if a > 0 else sp.latex(g)}}}^{{{sp.latex(g) if a > 0 else sp.latex(f)}}}\\left({sp.latex(z)}\\right)dx\\right)dy" +
                    f"\n\t=\ldots=\\int\\limits_{{{min(roz)}}}^{{{max(roz)}}}\\left({sp.latex(sp.integrate(z, (x, f, g)).expand()) if a > 0 else sp.latex(sp.integrate(z, (x, g, f)).expand())}\\right)dy")

            zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"
            return (
            f'Obliczyć ${zadanie}$ gdzie $D$ - obszar ograniczony krzywymi $x={sp.latex(f)}$, oraz $x={sp.latex(g)}$',
            f'\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}x}}\n' +
            f'\t\\includegraphics[width=45mm]{{../pics/calka_podwojna{nr_zadania}y}}\\\\\n' +
            '\tWzględem $Oy:$ $' + calka_y + ' = \ldots = ' + sp.latex(wynik_y) + '$ \\\\')



        return('','')

    else:
        return ('Nie ma takiego przypadku w całce podwójnej','')

if __name__ == "__main__":
    start = time.time()
    for i in range(1):
        polecenie, rozwiazanie = calka_podwojna(typ=2, nr_zadania=i)
        print(polecenie, '\n', rozwiazanie)
    print(f'czas generowania: {round(time.time() - start, 3)}s')



import random

import sympy as sp
from sympy.abc import n, x


def granica_ciagu(typ: int = 0):
    if typ == 0:
        liczby = (-3, -2, -1, 0, 0, 0, 1, 2, 3, 4, 5)
        while True:
            a, b, c, d, e, f, g, h, i, j = [random.choice(liczby) for _ in range(10)]
            if (a != d or b != e or c != f) and (a != 0 or b != 0 or c != 0 or d != 0) and (
                    e != 0 or f != 0 or j != 0):
                break
        granica = sp.limit(
            (a * n ** 3 + b * n ** 2 + c * n + d + i * n ** 5) / (e * n ** 3 + e * n ** 2 + f * n + g + j * n ** 5),
            n, sp.oo)
        return (f'Obliczyć granicę\n'
                f'\t\\[\n'
                f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}}'
                f' {sp.latex((a * n ** 3 + b * n ** 2 + c * n + d + i * n ** 5) / (e * n ** 3 + e * n ** 2 + f * n + g + j * n ** 5))} \n'
                f'\t\\]\n',
                f'${sp.latex(granica)}$')

    if typ == 1:
        liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
        gora = random.choice((0, 1))
        while True:
            a, b, c, d, e = [random.choice(liczby) for _ in range(5)]
            if gora == 1:
                granica = sp.limit((sp.sqrt(a * n ** 2 + b * n + c) / (d * n + e)), n, sp.oo)
                if granica.is_real:
                    return (f'Obliczyć granicę\n'
                            f'\t\\[\n'
                            f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}} {sp.latex((sp.sqrt(a * n ** 2 + b * n + c) / (d * n + e)))} \n'
                            f'\t\\]\n',
                            f'${sp.latex(granica)}$')
            else:
                granica = sp.limit((d * n + e) / (sp.sqrt(a * n ** 2 + b * n + c)), n, sp.oo)
                if granica.is_real:
                    return (f'Obliczyć granicę\n'
                            f'\t\\[\n'
                            f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}} {sp.latex((d * n + e) / (sp.sqrt(a * n ** 2 + b * n + c)))} \n'
                            f'\t\\]\n',
                            f'${sp.latex(granica)}$')

    if typ == 2:
        liczby = (-3, -2, -1, 1, 2, 3, 4)
        while True:
            a, b, c, d = [random.choice(liczby) for _ in range(4)]
            e, f, g, h = [random.choice((2, 3, 4, 5)) for _ in range(4)]
            if a != b and c != d and e != f and g != h:
                break
        granica = sp.limit((a * e ** n + b * f ** n) / (d * g ** n + e * h ** n), n, sp.oo)
        return (f'Obliczyć granicę\n'
                f'\t\\[\n'
                f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}} {sp.latex((a * e ** n + b * f ** n) / (d * g ** n + e * h ** n))} \n'
                f'\t\\]\n',
                f'${sp.latex(granica)}$')

    if typ == 3:
        liczby = (-3, -2, -1, 0, 1, 2, 3, 4, 5)
        while True:
            a, b, c, d, e, f = [random.choice(liczby) for _ in range(6)]
            if a == b > 0 and (c != d or e != f):
                break
        granica = sp.limit(sp.sqrt(a * n ** 2 + c * n + e) - sp.sqrt(b * n ** 2 + d * n + f), n, sp.oo)
        return (f'Obliczyć granicę\n'
                f'\t\\[\n'
                f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}} \\left({sp.latex(sp.sqrt(a * n ** 2 + c * n + e) - sp.sqrt(b * n ** 2 + d * n + f))} \\right) \n'
                f'\t\\]\n',
                f'${sp.latex(granica)}$')
    if typ == 4:
        liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
        while True:
            a, b, d, e, f = [random.choice(liczby) for _ in range(5)]
            c = a
            if b != d and e != 0:
                break
        granica = sp.limit(((a * n + b) / (c * n + d)) ** (e * n + f), n, sp.oo)
        return (f'Obliczyć granicę\n'
                f'\t\\[\n'
                f'\t\t\\lim\\limits_{{n \\rightarrow \\infty}} {sp.latex(((a * n + b) / (c * n + d)) ** (e * n + f))} \n'
                f'\t\\]\n',
                f'${sp.latex(granica)}$')
def granica_funkcji(typ: int = 0):
    if typ == 0:
        a, b, c = [random.choice((-2, -1, 1, 2, 3, 4)) for _ in range(3)]
        a = sp.Abs(a)
        granica = sp.limit((a - sp.sqrt(a ** 2 - b * x)) / (c * x), x, 0)
        return (f'Obliczyć granicę\n'
                f'\t\\[\n'
                f'\t\t\\lim\\limits_{{x \\rightarrow 0}}'
                f' {sp.latex((a - sp.sqrt(a**2-b*x))/(c*x))} \n'
                f'\t\\]\n',
                f'${sp.latex(granica)}$')

if __name__ == "__main__":
    polecenie, rozwiazanie = granica_funkcji(typ=0)
    print(polecenie, '\n', rozwiazanie)

    res2 = sp.limit(1 / x ** 2 - 1 / (x * sp.sin(x)), x, 0)

    print(res2)

    for i in range(1):
        granica = sp.limit(((sp.E ** (-2 / x ** 2)) ** (sp.sin(x ** 2))), x, 0)
        print(sp.latex(granica))



import os
import pickle
import random
import time
from pathlib import Path

import sympy as sp

sciezka = str(Path(__file__).parent)


def uklad_Cramera(wymiar: int = 3, gotowiec: bool = False):
    if gotowiec is True:
        if wymiar == 7:
            if os.path.isfile(sciezka + '//gotowe//uklad_Cramera_wymiar_7.pickle'):  # 10000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//uklad_Cramera_wymiar_7.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
        if wymiar == 6:
            if os.path.isfile(sciezka + '//gotowe//uklad_Cramera_wymiar_6.pickle'):  # 10000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//uklad_Cramera_wymiar_6.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    x, y, z, t, u, v, w = sp.symbols('x y z t u v w', real=True)
    X = sp.Matrix([x, y, z, t, u, v, w])
    X = sp.Matrix(X[:wymiar])
    liczby = (-3, -2, -1, 0, 1, 2, 3, 4) if wymiar >= 3 else (-3, -2, -1, 1, 2, 3, 4)
    niewiadoma = random.choice([i for i in range(wymiar)])
    while True:
        A = sp.Matrix(wymiar, wymiar, [sp.Rational(random.choice(liczby)) for _ in range(wymiar ** 2)])
        det_A = sp.det(A)
        # print(det_A)
        if det_A != 0 and det_A != 1 and abs(det_A) < 10:
            break
    while True:
        B = sp.Matrix(1, wymiar, [sp.Rational(random.choice(liczby)) for _ in range(wymiar)])
        A_temp = sp.ImmutableDenseMatrix(A)
        A_temp = A_temp.row_del(niewiadoma)
        A_temp = A_temp.row_insert(niewiadoma, B)
        det_A_temp = sp.det(A_temp)
        if det_A_temp != 0 and det_A_temp != 1 and abs(det_A_temp) < 10:
            break
    # Przestawienie alfabetyczne niewiadomych w tworzeniu zadania powoduje zmiany znaków wyznaczników
    # w przypadku t,x,y,z oraz t,u,v,x,y,z.
    if wymiar == 4 or wymiar == 6:
        det_A = -1 * det_A
        det_A_temp = -1 * det_A_temp
    # print(A,'\n',X,'\n', A_temp,'\n',B,'\n', det_A_temp/det_A,'\n',sp.latex(sp.Matrix.multiply(A.transpose(), X) - B.transpose()))
    lewa_strona = sp.Matrix.multiply(A.transpose(), X)
    uklad = ('\t\[\n'
             '\t\t\left\{\n'
             '\t\t\t\\begin{matrix}\n')
    for i in range(wymiar):
        uklad = uklad.join(['', f'\t\t\t\t{sp.latex(lewa_strona[i])} = {B[i]} \\\\ \n'])
    uklad = uklad.join(['', '\t\t\t\\end{matrix}\n\t\t\\right.\n\t\]'])
    # return uklad + str(X[niewiadoma]) + '=' + str(det_A_temp / det_A)

    return (f'Z układu równań wyznaczyć niewiadomą ${X[niewiadoma]}$\n' + uklad,
            f'$\\det(A) = {det_A},\\ \\det(A_{X[niewiadoma]})={det_A_temp},\\ '
            f'{X[niewiadoma]} = {sp.latex(det_A_temp / det_A)}$')


def uklad_rownan_nieoznaczony():  # na 10000 losowań żaden się nie powtórzył
    # random.seed()
    x, y, z, t = sp.symbols('x y z t', real=True)
    while True:
        stala_niewiadoma = random.choice([0, 1, 2, 3])
        wartosc_stalej_niewiadomej = random.choice([-2, -1, 2])
        while True:
            parametr = random.choice([1, 2, 3, 4])
            if parametr != stala_niewiadoma:
                break
        a = random.choice([-3, -2, -1, 2, 3, 4])
        b = random.choice([-3, -2, -1, 2, 3, 4])
        c = random.choice([-3, -2, -1, 2, 3, 4])
        d = random.choice([-3, -2, -1, 2, 3, 4])
        M = sp.zeros(5, )
        for ind in range(0, 3):
            M[ind, ind] = 1
        M[0, 3] = a
        M[0, 4] = b
        M[1, 3] = c
        M[1, 4] = d
        M[2, 4] = wartosc_stalej_niewiadomej
        for ind1 in range(0, 5):
            for ind2 in range(0, 5):
                if ind1 != ind2:
                    mnoznik = random.choice([-1, -1, 0, 1, 1])
                    for ind in range(0, 5):
                        M[ind2, ind] += M[ind1, ind] * mnoznik
        ile_zer = 0
        for ind1 in range(0, 5):
            for ind2 in range(0, 5):
                if M[ind1, ind2] == 0:
                    ile_zer += 1
        if max(abs(M)) < 10 and ile_zer == 2:
            break
    B = M.col(4)
    M.col_del(4)
    if stala_niewiadoma != 2:
        for ind in range(5):
            M[ind, stala_niewiadoma] += M[ind, 2]
            M[ind, 2] = M[ind, stala_niewiadoma] - M[ind, 2]
            M[ind, stala_niewiadoma] -= M[ind, 2]
    X = sp.Matrix([x, y, z, t])
    lewa_strona = sp.Matrix.multiply(M, X)
    uklad = ('\t\[\n'
             '\t\t\left\{\n'
             '\t\t\t\\begin{matrix}\n')
    for i in range(5):
        uklad = uklad.join(['', f'\t\t\t\t{sp.latex(lewa_strona[i])} = {B[i]} \\\\ \n'])
    uklad = uklad.join(['', '\t\t\t\\end{matrix}\n\t\t\\right.\n\t\]'])
    uklad1 = []
    for i in range(5):
        uklad1.append(str(lewa_strona[i]) + '-(' + str(B[i]) + ')')
    rozwiazanie = sp.latex(sp.solve(sp.sympify(uklad1))).replace(':', ' = ')
    odpowiedz = (f'${rozwiazanie}$')
    return ('Rozwiązać układ równań.\\ Jeśli możliwe podać trzy przykładowe rozwiązania.\\ '
            'Jedno rozwiązanie sprawdzić. \n' + uklad,
            odpowiedz)


if __name__ == "__main__":  # to się uruchamia tylko gdy plik jest uruchamiany jako program a nie ładowany jako moduł
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    start_time = time.time()
    # print('!!! uklady_rownan_liniowych.py były planowane jako moduł a uruchomiłeś je jako skrypt !!!')
    # print(rownanie_liniowe())
    # print(rownanie_kwadratowe())
    # polecenie, rozwiazanie = uklad_rownan_nieoznaczony()
    polecenie, rozwiazanie = uklad_Cramera(wymiar=random.choice([6]), gotowiec=True)
    # polecenie, rozwiazanie = uklad_Cramera(wymiar=4)
    print(polecenie, '\n', rozwiazanie)
    print("--- %s seconds ---" % (time.time() - start_time))



import datetime  # do pobierania daty
import textwrap

import babel.dates  # do ładnego generowania daty


def dodaj_zadanie(zadanie, warstwa):
    polecenie, rozwiazanie = zadanie
    polecenie = '\\item ' + polecenie + '\n'
    polecenie = textwrap.indent(polecenie, prefix='\t\t')
    rozwiazanie = f'\\zOdpowiedziami{{\\kolorodpowiedzi}}{{ocg{warstwa}}}\n\t{{{rozwiazanie}}}\n\n'
    rozwiazanie = textwrap.indent(rozwiazanie, prefix='\t\t\t')
    return polecenie + rozwiazanie


def generuj_LaTeX(kierunek,
                  grupa,
                  przedmiot,
                  semestr,
                  kolokwium,
                  data,
                  kolor_odpowiedzi,
                  *zadania):
    plik_tex = ('% !TeX spellcheck = pl_PL-Polish\n'
                '\\documentclass[a4paper,12pt]{article}\n'  # można zmieniać rozmiar czcionki
                '\\linespread{1.3} %odstepy miedzy liniami\n'
                '\\usepackage[a4paper, lmargin=2cm, rmargin=2cm, tmargin=2cm, bmargin=2cm]{geometry}\n'
                '\\usepackage{amsfonts}\n'
                '\\usepackage{amsmath}\n'
                '\\usepackage{color}\n'
                '\\usepackage{enumitem}\n'
                '\\usepackage{fancyhdr}\n'
                '\\usepackage{float}\n'
                '\\usepackage{graphicx}\n'  # do pdf
                '\\usepackage{ifthen}\n'
                '\\usepackage[utf8]{inputenc}\n'
                '\\usepackage{lmodern}\n'
                '\\usepackage{ocgx}\n'
                # '\\usepackage{pgf}\n'  # niepotrzebne gdy importujemy pdf-y
                '\\usepackage{polski}\n'
                '\\usepackage{tcolorbox}\n'
                '\\tcbuselibrary{most}\n'
                '\\tcbuselibrary{skins}\n'
                '\\tcbuselibrary{raster}\n'
                '% brak - bez odpowiedzi i bez miejsca, white - bez odpowiedzi z miejscem, red = odpowiedzi ukryte ale dostepne\n'
                f'\\newcommand{{\kolorodpowiedzi}}{{{kolor_odpowiedzi}}}\n'
                '\\renewcommand{\\footrulewidth}{0.4pt}% linia pozioma na końcu strony - default is 0pt\n'
                '\\DeclareFontShape{OMX}{cmex}{m}{n}\n'
                '    {<-7.5> cmex7\n'
                '    <7.5-8.5> cmex8\n'
                '    <8.5-9.5> cmex9\n'
                '    <9.5-> cmex10}{}\n'
                '\\DeclareSymbolFont{largesymbols}{OMX}{cmex}{m}{n}\n\n'
                # '\\newcommand{\\zOdpowiedziami}[1]{\n'
                # '\t\\ifthenelse{\\equal{#1}{brak}}{\\hphantom}{\n'
                # '\t\t\\ifthenelse{\\equal{#1}{white}}{\\vphantom}{\n'
                # '\t\t\t\\textcolor{\\kolorodpowiedzi}}}}\n\n'
                '\n'
                '\\newcommand{\\ukryte}{1}  % domyślnie odpowiedzi są do pokazywania po kliknięciu\n'
                '\\ifthenelse{\\equal{\\kolorodpowiedzi}{red}}  % ukrywamy od pokazywania gdy kolor jest red\n'
                '\t{\\renewcommand{\\ukryte}{0}}{}\n\n'
                '\\newcommand{\\zOdpowiedziami}[3]{\n'
                '\t\\ifthenelse{\\equal{#1}{brak}}{}{\n'
                '\t\t\\ifthenelse{\\equal{#1}{white}}{\\vphantom{#3}}{\n'
                '\t\t\t\\switchocg{#2}{\\textcolor{\\kolorodpowiedzi}{\\\\Rozwiązanie: }}\n'
                '\t\t\t\t\\begin{ocg}{Warstwa odpowiedzi}{#2}{\\ukryte}\n'
                '\t\t\t\t\t\\textcolor{\kolorodpowiedzi}{#3}\n'
                '\t\t\t\t\\end{ocg}}}}\n\n'
    
                '\\pdfinfo{/Author (https://generator-zadan.readthedocs.io)}\n\n'

                '\\begin{document}\n'
                '    \\pagestyle{fancy}\n'
                '    \\setlength{\headheight}{27.29453pt}\n'
                '    \\fancyhead{}\n'
                '    \\fancyhead[L]{\\textbf{' + kierunek + '\\\\' + przedmiot + ' - ' + grupa + '}}\n'
                                                                                                 '    \\fancyhead[R]{\\textbf{' + kolokwium + '\\\\ ' + data + '}}' + '\n' +
                '    \\fancyfoot{}' + '\n' +
                '    \\fancyfoot[R]{\\tiny\\textbf{' +
                babel.dates.format_datetime(datetime.datetime.now(), "d MMMM yyyy, HH:mm", locale='pl_PL') + '}}\n' +
                '%    \hspace{1cm}\n' +
                r'    \begin{enumerate}[label= \textbf{Zadanie \arabic*. },' +
                ' leftmargin=1cm, align=left, itemsep=0pt]' + '\n'
                )
    for n, zadanie in enumerate(zadania):
        plik_tex += dodaj_zadanie(zadanie, n)

    plik_tex += (r'    \end{enumerate}' + '\n'
                 + '\end{document}')
    return plik_tex
